@@ -156,10 +156,10 @@ void branch_new(void){
   }
   db_multi_exec("INSERT OR IGNORE INTO unsent VALUES(%d)", brid);
   if( manifest_crosslink(brid, &branch, MC_PERMIT_HOOKS)==0 ){
-    fossil_fatal("%s\n", g.zErrMsg);
+    fossil_fatal("%s", g.zErrMsg);
   }
   assert( blob_is_reset(&branch) );
-  content_deltify(rootid, brid, 0);
+  content_deltify(rootid, &brid, 1, 0);
   zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", brid);
   fossil_print("New branch: %s\n", zUuid);
   if( g.argc==3 ){
@@ -190,7 +190,7 @@ void branch_new(void){
 #define BRL_BOTH             0x003 /* Show both open and closed branches */
 #define BRL_OPEN_CLOSED_MASK 0x003
 #define BRL_MTIME            0x004 /* Include lastest check-in time */
-#dfeine BRL_ORDERBY_MTIME    0x008 /* Sort by MTIME. (otherwise sort by name)*/
+#define BRL_ORDERBY_MTIME    0x008 /* Sort by MTIME. (otherwise sort by name)*/
 
 #endif /* INTERFACE */
 
@@ -241,6 +241,26 @@ void branch_prepare_list_query(Stmt *pQuery, int brFlags){
   }
 }
 
+/*
+** If the branch named in the argument is open, return a RID for one of
+** the open leaves of that branch.  If the branch does not exists or is
+** closed, return 0.
+*/
+int branch_is_open(const char *zBrName){
+  return db_int(0,
+    "SELECT rid FROM tagxref AS ox"
+    " WHERE tagid=%d"
+    "   AND tagtype=2"
+    "   AND value=%Q"
+    "   AND rid IN leaf"
+    "   AND NOT EXISTS(SELECT 1 FROM tagxref AS ix"
+                      " WHERE tagid=%d"
+                      "   AND tagtype=1"
+                      "   AND ox.rid=ix.rid)",
+    TAG_BRANCH, zBrName, TAG_CLOSED
+  );
+}
+
 
 /*
 ** COMMAND: branch
@@ -250,7 +270,7 @@ void branch_prepare_list_query(Stmt *pQuery, int brFlags){
 ** Run various subcommands to manage branches of the open repository or
 ** of the repository identified by the -R or --repository option.
 **
-**    %fossil branch new BRANCH-NAME BASIS ?OPTIONS?
+**    fossil branch new BRANCH-NAME BASIS ?OPTIONS?
 **
 **        Create a new branch BRANCH-NAME off of check-in BASIS.
 **        Supported options for this subcommand include:
@@ -260,13 +280,22 @@ void branch_prepare_list_query(Stmt *pQuery, int brFlags){
 **        --date-override DATE  DATE to use instead of 'now'
 **        --user-override USER  USER to use instead of the current default
 **
-**    %fossil branch list ?-a|--all|-c|--closed?
-**    %fossil branch ls ?-a|--all|-c|--closed?
+**        DATE may be "now" or "YYYY-MM-DDTHH:MM:SS.SSS". If in
+**        year-month-day form, it may be truncated, the "T" may be
+**        replaced by a space, and it may also name a timezone offset
+**        from UTC as "-HH:MM" (westward) or "+HH:MM" (eastward).
+**        Either no timezone suffix or "Z" means UTC.
+**
+**    fossil branch list|ls ?-a|--all|-c|--closed?
 **
 **        List all branches.  Use -a or --all to list all branches and
 **        -c or --closed to list all closed branches.  The default is to
 **        show only open branches.
 **
+**    fossil branch info BRANCH-NAME
+**
+**        Print information about a branch
+**
 ** Options:
 **    -R|--repository FILE       Run commands on repository FILE
 */
@@ -298,9 +327,24 @@ void branch_cmd(void){
       fossil_print("%s%s\n", (isCur ? "* " : "  "), zBr);
     }
     db_finalize(&q);
+  }else if( strncmp(zCmd,"info",n)==0 ){
+    int i;
+    for(i=3; i<g.argc; i++){
+      const char *zBrName = g.argv[i];
+      int rid = branch_is_open(zBrName);
+      if( rid==0 ){
+        fossil_print("%s: not an open branch\n", zBrName);
+      }else{
+        const char *zUuid = db_text(0,"SELECT uuid FROM blob WHERE rid=%d",rid);
+        const char *zDate = db_text(0,
+          "SELECT datetime(mtime,toLocal()) FROM event"
+          " WHERE objid=%d", rid);
+        fossil_print("%s: open as of %s on %.16s\n", zBrName, zDate, zUuid);
+      }
+    }
   }else{
     fossil_fatal("branch subcommand should be one of: "
-                 "new list ls");
+                 "info list ls new");
   }
 }
 
@@ -319,7 +363,8 @@ static const char brlistQuery[] =
 @      AND tagxref.tagid=(SELECT tagid FROM tag WHERE tagname='branch')
 @      AND tagtype>0),
 @   count(*),
-@   (SELECT uuid FROM blob WHERE rid=tagxref.rid)
+@   (SELECT uuid FROM blob WHERE rid=tagxref.rid),
+@   event.bgcolor
 @  FROM tagxref, tag, event
 @ WHERE tagxref.tagid=tag.tagid
 @   AND tagxref.tagtype>0
@@ -340,10 +385,12 @@ static const char brlistQuery[] =
 static void new_brlist_page(void){
   Stmt q;
   double rNow;
+  int show_colors = PB("colors");
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
   style_header("Branches");
   style_adunit_config(ADUNIT_RIGHT_OK);
+  style_submenu_checkbox("colors", "Use Branch Colors", 0, 0);
   login_anonymous_available();
 
   db_prepare(&q, brlistQuery/*works-like:""*/);
@@ -363,10 +410,22 @@ static void new_brlist_page(void){
     const char *zMergeTo = db_column_text(&q, 3);
     int nCkin = db_column_int(&q, 4);
     const char *zLastCkin = db_column_text(&q, 5);
+    const char *zBgClr = db_column_text(&q, 6);
     char *zAge = human_readable_age(rNow - rMtime);
     sqlite3_int64 iMtime = (sqlite3_int64)(rMtime*86400.0);
     if( zMergeTo && zMergeTo[0]==0 ) zMergeTo = 0;
-    @ <tr>
+    if( zBgClr == 0 ){
+      if( zBranch==0 || strcmp(zBranch,"trunk")==0 ){
+        zBgClr = 0;
+      }else{
+        zBgClr = hash_color(zBranch);
+      }
+    }
+    if( zBgClr && zBgClr[0] && show_colors ){
+      @ <tr style="background-color:%s(zBgClr)">
+    }else{
+      @ <tr>
+    }
     @ <td>%z(href("%R/timeline?n=100&r=%T",zBranch))%h(zBranch)</a></td>
     @ <td data-sortkey="%016llx(-iMtime)">%s(zAge)</td>
     @ <td>%d(nCkin)</td>
@@ -423,21 +482,21 @@ void brlist_page(void){
 
   style_header("%s", showClosed ? "Closed Branches" :
                         showAll ? "All Branches" : "Open Branches");
-  style_submenu_element("Timeline", "Timeline", "brtimeline");
+  style_submenu_element("Timeline", "brtimeline");
   if( showClosed ){
-    style_submenu_element("All", "All", "brlist?all");
-    style_submenu_element("Open","Open","brlist?open");
+    style_submenu_element("All", "brlist?all");
+    style_submenu_element("Open", "brlist?open");
   }else if( showAll ){
-    style_submenu_element("Closed", "Closed", "brlist?closed");
-    style_submenu_element("Open","Open","brlist");
+    style_submenu_element("Closed", "brlist?closed");
+    style_submenu_element("Open", "brlist");
   }else{
-    style_submenu_element("All", "All", "brlist?all");
-    style_submenu_element("Closed","Closed","brlist?closed");
+    style_submenu_element("All", "brlist?all");
+    style_submenu_element("Closed", "brlist?closed");
   }
   if( !colorTest ){
-    style_submenu_element("Color-Test", "Color-Test", "brlist?colortest");
+    style_submenu_element("Color-Test", "brlist?colortest");
   }else{
-    style_submenu_element("All", "All", "brlist?all");
+    style_submenu_element("All", "brlist?all");
   }
   login_anonymous_available();
 #if 0
@@ -525,7 +584,7 @@ void brtimeline_page(void){
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
 
   style_header("Branches");
-  style_submenu_element("List", "List", "brlist");
+  style_submenu_element("List", "brlist");
   login_anonymous_available();
   @ <h2>The initial check-in for each branch:</h2>
   db_prepare(&q,
