@@ -173,36 +173,37 @@ void page_dir(void){
     blob_append(&dirname, "in directory ", -1);
     hyperlinked_path(zD, &dirname, zCI, "dir", "");
     zPrefix = mprintf("%s/", zD);
-    style_submenu_element("Top-Level", "Top-Level", "%s",
+    style_submenu_element("Top-Level", "%s",
                           url_render(&sURI, "name", 0, 0, 0));
   }else{
     blob_append(&dirname, "in the top-level directory", -1);
     zPrefix = "";
   }
   if( linkTrunk ){
-    style_submenu_element("Trunk", "Trunk", "%s",
+    style_submenu_element("Trunk", "%s",
                           url_render(&sURI, "ci", "trunk", 0, 0));
   }
   if( linkTip ){
-    style_submenu_element("Tip", "Tip", "%s",
-                          url_render(&sURI, "ci", "tip", 0, 0));
+    style_submenu_element("Tip", "%s", url_render(&sURI, "ci", "tip", 0, 0));
   }
   if( zCI ){
     @ <h2>Files of check-in [%z(href("vinfo?name=%!S",zUuid))%S(zUuid)</a>]
-    @ %s(blob_str(&dirname))</h2>
+    @ %s(blob_str(&dirname))
+    if( zD ){
+      @ &nbsp;&nbsp;%z(href("%R/timeline?chng=%T/*", zD))[history]</a>
+    }
+    @ </h2>
     zSubdirLink = mprintf("%R/dir?ci=%!S&name=%T", zUuid, zPrefix);
     if( nD==0 ){
-      style_submenu_element("File Ages", "File Ages", "%R/fileage?name=%!S",
-                            zUuid);
+      style_submenu_element("File Ages", "%R/fileage?name=%!S", zUuid);
     }
   }else{
     @ <h2>The union of all files from all check-ins
     @ %s(blob_str(&dirname))</h2>
     zSubdirLink = mprintf("%R/dir?name=%T", zPrefix);
   }
-  style_submenu_element("All", "All", "%s",
-                        url_render(&sURI, "ci", 0, 0, 0));
-  style_submenu_element("Tree-View", "Tree-View", "%s",
+  style_submenu_element("All", "%s", url_render(&sURI, "ci", 0, 0, 0));
+  style_submenu_element("Tree-View", "%s",
                         url_render(&sURI, "type", "tree", 0, 0));
 
   /* Compute the temporary table "localfiles" containing the names
@@ -300,6 +301,62 @@ void page_dir(void){
   db_finalize(&q);
   manifest_destroy(pM);
   @ </ul></td></tr></table>
+
+  /* If the directory contains a readme file, then display its content below
+  ** the list of files
+  */
+  db_prepare(&q,
+    "SELECT x, u FROM localfiles"
+    " WHERE x COLLATE nocase IN"
+    " ('readme','readme.txt','readme.md','readme.wiki','readme.markdown',"
+    " 'readme.html') ORDER BY x LIMIT 1;"
+  );
+  if( db_step(&q)==SQLITE_ROW ){
+    const char *zName = db_column_text(&q,0);
+    const char *zUuid = db_column_text(&q,1);
+    if( zUuid ){
+      rid = fast_uuid_to_rid(zUuid);
+    }else{
+      if( zD ){
+        rid = db_int(0,
+           "SELECT fid FROM filename, mlink, event"
+           " WHERE name='%q/%q'"
+           "   AND mlink.fnid=filename.fnid"
+           "   AND event.objid=mlink.mid"
+           " ORDER BY event.mtime DESC LIMIT 1",
+           zD, zName
+        );
+      }else{
+        rid = db_int(0,
+           "SELECT fid FROM filename, mlink, event"
+           " WHERE name='%q'"
+           "   AND mlink.fnid=filename.fnid"
+           "   AND event.objid=mlink.mid"
+           " ORDER BY event.mtime DESC LIMIT 1",
+           zName
+        );
+      }
+    }
+    if( rid ){
+      @ <hr>
+      if( sqlite3_strlike("readme.html", zName, 0)==0 ){
+        if( zUuid==0 ){
+          zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", rid);
+        }
+        @ <iframe src="%R/raw/%s(zUuid)"
+        @ width="100%%" frameborder="0" marginwidth="0" marginheight="0"
+        @ sandbox="allow-same-origin"
+        @ onload="this.height=this.contentDocument.documentElement.scrollHeight;">
+        @ </iframe>
+      }else{
+        Blob content;
+        const char *zMime = mimetype_from_name(zName);
+        content_get(rid, &content);
+        wiki_render_by_mimetype(&content, zMime);
+      }
+    }
+  }
+  db_finalize(&q);
   style_footer();
 }
 
@@ -320,7 +377,7 @@ struct FileTreeNode {
   FileTreeNode *pLastChild; /* Last child on the pChild list */
   char *zName;              /* Name of this entry.  The "tail" */
   char *zFullName;          /* Full pathname of this entry */
-  char *zUuid;              /* SHA1 hash of this file.  May be NULL. */
+  char *zUuid;              /* Artifact hash of this file.  May be NULL. */
   double mtime;             /* Modification time for this entry */
   unsigned nFullName;       /* Length of zFullName */
   unsigned iLevel;          /* Levels of parent directories */
@@ -350,7 +407,7 @@ struct FileTree {
 static void tree_add_node(
   FileTree *pTree,         /* Tree into which nodes are added */
   const char *zPath,       /* The full pathname of file to add */
-  const char *zUuid,       /* UUID of the file.  Might be NULL. */
+  const char *zUuid,       /* Hash of the file.  Might be NULL. */
   double mtime             /* Modification time for this entry */
 ){
   int i;
@@ -373,7 +430,7 @@ static void tree_add_node(
     int nByte;
     while( zPath[i] && zPath[i]!='/' ){ i++; }
     nByte = sizeof(*pNew) + i + 1;
-    if( zUuid!=0 && zPath[i]==0 ) nByte += UUID_SIZE+1;
+    if( zUuid!=0 && zPath[i]==0 ) nByte += HNAME_MAX+1;
     pNew = fossil_malloc( nByte );
     memset(pNew, 0, sizeof(*pNew));
     pNew->zFullName = (char*)&pNew[1];
@@ -382,7 +439,7 @@ static void tree_add_node(
     pNew->nFullName = i;
     if( zUuid!=0 && zPath[i]==0 ){
       pNew->zUuid = pNew->zFullName + i + 1;
-      memcpy(pNew->zUuid, zUuid, UUID_SIZE+1);
+      memcpy(pNew->zUuid, zUuid, strlen(zUuid)+1);
     }
     pNew->zName = pNew->zFullName + iStart;
     if( pTree->pLast ){
@@ -618,7 +675,7 @@ void page_tree(void){
     blob_append(&dirname, "within directory ", -1);
     hyperlinked_path(zD, &dirname, zCI, "tree", zREx);
     if( zRE ) blob_appendf(&dirname, " matching \"%s\"", zRE);
-    style_submenu_element("Top-Level", "Top-Level", "%s",
+    style_submenu_element("Top-Level", "%s",
                           url_render(&sURI, "name", 0, 0, 0));
   }else{
     if( zRE ){
@@ -627,22 +684,19 @@ void page_tree(void){
   }
   style_submenu_binary("mtime","Sort By Time","Sort By Filename", 0);
   if( zCI ){
-    style_submenu_element("All", "All", "%s",
-                          url_render(&sURI, "ci", 0, 0, 0));
+    style_submenu_element("All", "%s", url_render(&sURI, "ci", 0, 0, 0));
     if( nD==0 && !showDirOnly ){
-      style_submenu_element("File Ages", "File Ages", "%R/fileage?name=%s",
-                            zUuid);
+      style_submenu_element("File Ages", "%R/fileage?name=%s", zUuid);
     }
   }
   if( linkTrunk ){
-    style_submenu_element("Trunk", "Trunk", "%s",
+    style_submenu_element("Trunk", "%s",
                           url_render(&sURI, "ci", "trunk", 0, 0));
   }
   if( linkTip ){
-    style_submenu_element("Tip", "Tip", "%s",
-                          url_render(&sURI, "ci", "tip", 0, 0));
+    style_submenu_element("Tip", "%s", url_render(&sURI, "ci", "tip", 0, 0));
   }
-  style_submenu_element("Flat-View", "Flat-View", "%s",
+  style_submenu_element("Flat-View", "%s",
                         url_render(&sURI, "type", "flat", 0, 0));
 
   /* Compute the file hierarchy.
@@ -697,14 +751,12 @@ void page_tree(void){
       if( p->pChild!=0 && p->nFullName>nD ) nFile++;
     }
     zObjType = "Folders";
-    style_submenu_element("Files","Files","%s",
-                          url_render(&sURI,"nofiles",0,0,0));
   }else{
     zObjType = "Files";
-    style_submenu_element("Folders","Folders","%s",
-                          url_render(&sURI,"nofiles","1",0,0));
   }
 
+  style_submenu_checkbox("nofiles", "Folders Only", 0, 0);
+
   if( zCI ){
     @ <h2>%s(zObjType) from
     if( sqlite3_strnicmp(zCI, zUuid, (int)strlen(zCI))!=0 ){
@@ -1034,27 +1086,25 @@ void fileage_page(void){
   baseTime = db_double(0.0,"SELECT mtime FROM event WHERE objid=%d", rid);
   zNow = db_text("", "SELECT datetime(mtime,toLocal()) FROM event"
                      " WHERE objid=%d", rid);
-  style_submenu_element("Tree-View", "Tree-View",
-                        "%R/tree?ci=%T&mtime=1&type=tree",
-                        zName);
+  style_submenu_element("Tree-View", "%R/tree?ci=%T&mtime=1&type=tree", zName);
   style_header("File Ages");
   zGlob = P("glob");
   compute_fileage(rid,zGlob);
   db_multi_exec("CREATE INDEX fileage_ix1 ON fileage(mid,pathname);");
 
-  @ <h2>Files in
+  @ <h1>Files in
   @ %z(href("%R/info/%!S",zUuid))[%S(zUuid)]</a>
   if( zGlob && zGlob[0] ){
-    @ that match "%h(zGlob)" and
+    @ that match "%h(zGlob)"
   }
-  @ ordered by check-in time</h2>
+  @ ordered by age</h1>
   @
-  @ <p>Times are relative to the check-in time for
-  @ %z(href("%R/ci/%!S",zUuid))[%S(zUuid)]</a> which is
+  @ <p>File ages are expressed relative to the
+  @ %z(href("%R/ci/%!S",zUuid))[%S(zUuid)]</a> check-in time of
   @ %z(href("%R/timeline?c=%t",zNow))%s(zNow)</a>.</p>
   @
   @ <div class='fileage'><table>
-  @ <tr><th>Time</th><th>Files</th><th>Check-in</th></tr>
+  @ <tr><th>Age</th><th>Files</th><th>Check-in</th></tr>
   db_prepare(&q1,
     "SELECT event.mtime, event.objid, blob.uuid,\n"
     "       coalesce(event.ecomment,event.comment),\n"
@@ -1090,9 +1140,9 @@ void fileage_page(void){
       const char *zFile = db_column_text(&q2,1);
       int fid = db_column_int(&q2,2);
       if( showId ){
-        @ %z(href("%R/artifact/%!S",zFUuid))%h(zFile)</a> (%d(fid))<br>
+        @ %z(href("%R/artifact/%!S",zFUuid))%h(zFile)</a> (%d(fid))<br />
       }else{
-        @ %z(href("%R/artifact/%!S",zFUuid))%h(zFile)</a><br>
+        @ %z(href("%R/artifact/%!S",zFUuid))%h(zFile)</a><br />
       }
     }
     db_reset(&q2);
