@@ -101,7 +101,7 @@ void delete_private_content(void){
 ** Note 1: For ssh and filesystem, path must have an extra leading
 **         '/' to use an absolute path.
 **
-** Note 2: Use %HH escapes for special characters in the userid and 
+** Note 2: Use %HH escapes for special characters in the userid and
 **         password.  For example "%40" in place of "@", "%2f" in place
 **         of "/", and "%3a" in place of ":".
 **
@@ -116,7 +116,8 @@ void delete_private_content(void){
 **    --ssl-identity FILENAME    Use the SSL identity if requested by the server
 **    --ssh-command|-c SSH       Use SSH as the "ssh" command
 **    --httpauth|-B USER:PASS    Add HTTP Basic Authorization to requests
-**    --verbose                  Show more statistics in output
+**    -u|--unversioned           Also sync unversioned content
+**    -v|--verbose               Show more statistics in output
 **
 ** See also: init
 */
@@ -131,7 +132,8 @@ void clone_cmd(void){
   /* Also clone private branches */
   if( find_option("private",0,0)!=0 ) syncFlags |= SYNC_PRIVATE;
   if( find_option("once",0,0)!=0) urlFlags &= ~URL_REMEMBER;
-  if( find_option("verbose",0,0)!=0) syncFlags |= SYNC_VERBOSE;
+  if( find_option("verbose","v",0)!=0) syncFlags |= SYNC_VERBOSE;
+  if( find_option("unversioned","u",0)!=0 ) syncFlags |= SYNC_UNVERSIONED;
   zHttpAuth = find_option("httpauth","B",1);
   zDefaultUser = find_option("admin-user","A",1);
   clone_ssh_find_options();
@@ -175,6 +177,7 @@ void clone_cmd(void){
     db_set("content-schema", CONTENT_SCHEMA, 0);
     db_set("aux-schema", AUX_SCHEMA_MAX, 0);
     db_set("rebuilt", get_version(), 0);
+    db_unset("hash-policy", 0);
     remember_or_get_http_auth(zHttpAuth, urlFlags & URL_REMEMBER, g.argv[2]);
     url_remember();
     if( g.zSSLIdentity!=0 ){
@@ -292,3 +295,48 @@ void clone_ssh_db_set_options(void){
     db_set("ssh-command", g.zSshCmd, 0);
   }
 }
+
+/*
+** WEBPAGE: download
+**
+** Provide a simple page that enables newbies to download the latest tarball or
+** ZIP archive, and provides instructions on how to clone.
+*/
+void download_page(void){
+  login_check_credentials();
+  style_header("Download Page");
+  if( !g.perm.Zip ){
+    @ <p>Bummer.  You do not have permission to download.
+    if( g.zLogin==0 || g.zLogin[0]==0 ){
+      @ Maybe it would work better if you
+      @ <a href="../login">logged in</a>.
+    }else{
+      @ Contact the site administrator and ask them to give
+      @ you "Download Zip" privileges.
+    }
+  }else{
+    const char *zDLTag = db_get("download-tag","trunk");
+    const char *zNm = db_get("short-project-name","download");
+    char *zUrl = href("%R/zip/%t.zip?uuid=%t", zNm, zDLTag);
+    @ <p>ZIP Archive: %z(zUrl)%h(zNm).zip</a>
+    zUrl = href("%R/tarball/%t.tar.gz?uuid=%t", zNm, zDLTag);
+    @ <p>Tarball: %z(zUrl)%h(zNm).tar.gz</a>
+  }
+  if( !g.perm.Clone ){
+    @ <p>You are not authorized to clone this repository.
+    if( g.zLogin==0 || g.zLogin[0]==0 ){
+      @ Maybe you would be able to clone if you
+      @ <a href="../login">logged in</a>.
+    }else{
+      @ Contact the site administrator and ask them to give
+      @ you "Clone" privileges in order to clone.
+    }
+  }else{
+    const char *zNm = db_get("short-project-name","clone");
+    @ <p>Clone the repository using this command:
+    @ <blockquote><pre>
+    @ fossil  clone  %s(g.zBaseURL)  %h(zNm).fossil
+    @ </pre></blockquote>
+  }
+  style_footer();
+}
