@@ -109,7 +109,7 @@ static int file_dir_match(FileDirList *p, const char *zFile){
 ** Print the "Index:" message that patches wants to see at the top of a diff.
 */
 void diff_print_index(const char *zFile, u64 diffFlags){
-  if( (diffFlags & (DIFF_SIDEBYSIDE|DIFF_BRIEF))==0 ){
+  if( (diffFlags & (DIFF_SIDEBYSIDE|DIFF_BRIEF|DIFF_NUMSTAT))==0 ){
     char *z = mprintf("Index: %s\n%.66c\n", zFile, '=');
     fossil_print("%s", z);
     fossil_free(z);
@@ -154,6 +154,9 @@ void diff_print_filenames(const char *zLeft, const char *zRight, u64 diffFlags){
 ** The difference is the set of edits needed to transform pFile1 into
 ** zFile2.  The content of pFile1 is in memory.  zFile2 exists on disk.
 **
+** If fSwapDiff is 1, show the set of edits to transform zFile2 into pFile1
+** instead of the opposite.
+**
 ** Use the internal diff logic if zDiffCmd is NULL.  Otherwise call the
 ** command zDiffCmd to do the diffing.
 **
@@ -169,7 +172,8 @@ void diff_file(
   const char *zDiffCmd,     /* Command for comparison */
   const char *zBinGlob,     /* Treat file names matching this as binary */
   int fIncludeBinary,       /* Include binary files for external diff */
-  u64 diffFlags             /* Flags to control the diff */
+  u64 diffFlags,            /* Flags to control the diff */
+  int fSwapDiff             /* Diff from Zfile2 to Pfile1 */
 ){
   if( zDiffCmd==0 ){
     Blob out;                 /* Diff output text */
@@ -196,10 +200,18 @@ void diff_file(
       }
     }else{
       blob_zero(&out);
-      text_diff(pFile1, &file2, &out, 0, diffFlags);
+      if( fSwapDiff ){
+        text_diff(&file2, pFile1, &out, 0, diffFlags);
+      }else{
+        text_diff(pFile1, &file2, &out, 0, diffFlags);
+      }
       if( blob_size(&out) ){
-        diff_print_filenames(zName, zName2, diffFlags);
-        fossil_print("%s\n", blob_str(&out));
+        if( diffFlags & DIFF_NUMSTAT ){
+          fossil_print("%s %s\n", blob_str(&out), zName);
+        }else{
+          diff_print_filenames(zName, zName2, diffFlags);
+          fossil_print("%s\n", blob_str(&out));
+        }
       }
       blob_reset(&out);
     }
@@ -253,10 +265,14 @@ void diff_file(
 
     /* Construct the external diff command */
     blob_zero(&cmd);
-    blob_appendf(&cmd, "%s ", zDiffCmd);
-    shell_escape(&cmd, blob_str(&nameFile1));
-    blob_append(&cmd, " ", 1);
-    shell_escape(&cmd, zFile2);
+    blob_append(&cmd, zDiffCmd, -1);
+    if( fSwapDiff ){
+      blob_append_escaped_arg(&cmd, zFile2);
+      blob_append_escaped_arg(&cmd, blob_str(&nameFile1));
+    }else{
+      blob_append_escaped_arg(&cmd, blob_str(&nameFile1));
+      blob_append_escaped_arg(&cmd, zFile2);
+    }
 
     /* Run the external diff command */
     fossil_system(blob_str(&cmd));
@@ -298,15 +314,21 @@ void diff_file_mem(
 
     blob_zero(&out);
     text_diff(pFile1, pFile2, &out, 0, diffFlags);
-    diff_print_filenames(zName, zName, diffFlags);
-    fossil_print("%s\n", blob_str(&out));
+    if( diffFlags & DIFF_NUMSTAT ){
+      fossil_print("%s %s\n", blob_str(&out), zName);
+    }else{
+      diff_print_filenames(zName, zName, diffFlags);
+      fossil_print("%s\n", blob_str(&out));
+    }
 
     /* Release memory resources */
     blob_reset(&out);
   }else{
     Blob cmd;
-    char zTemp1[300];
-    char zTemp2[300];
+    Blob temp1;
+    Blob temp2;
+    Blob prefix1;
+    Blob prefix2;
 
     if( !fIncludeBinary ){
       if( isBin1 || isBin2 ){
@@ -324,25 +346,35 @@ void diff_file_mem(
       }
     }
 
+    /* Construct a prefix for the temporary file names */
+    blob_zero(&prefix1);
+    blob_zero(&prefix2);
+    blob_appendf(&prefix1, "%s-v1", zName);
+    blob_appendf(&prefix2, "%s-v2", zName);
+
     /* Construct a temporary file names */
-    file_tempname(sizeof(zTemp1), zTemp1);
-    file_tempname(sizeof(zTemp2), zTemp2);
-    blob_write_to_file(pFile1, zTemp1);
-    blob_write_to_file(pFile2, zTemp2);
+    file_tempname(&temp1, blob_str(&prefix1));
+    file_tempname(&temp2, blob_str(&prefix2));
+    blob_write_to_file(pFile1, blob_str(&temp1));
+    blob_write_to_file(pFile2, blob_str(&temp2));
 
     /* Construct the external diff command */
     blob_zero(&cmd);
-    blob_appendf(&cmd, "%s ", zDiffCmd);
-    shell_escape(&cmd, zTemp1);
-    blob_append(&cmd, " ", 1);
-    shell_escape(&cmd, zTemp2);
+    blob_append(&cmd, zDiffCmd, -1);
+    blob_append_escaped_arg(&cmd, blob_str(&temp1));
+    blob_append_escaped_arg(&cmd, blob_str(&temp2));
 
     /* Run the external diff command */
     fossil_system(blob_str(&cmd));
 
     /* Delete the temporary file and clean up memory used */
-    file_delete(zTemp1);
-    file_delete(zTemp2);
+    file_delete(blob_str(&temp1));
+    file_delete(blob_str(&temp2));
+
+    blob_reset(&prefix1);
+    blob_reset(&prefix2);
+    blob_reset(&temp1);
+    blob_reset(&temp2);
     blob_reset(&cmd);
   }
 }
@@ -371,8 +403,10 @@ static void diff_against_disk(
   Blob sql;
   Stmt q;
   int asNewFile;            /* Treat non-existant files as empty files */
+  int isNumStat;            /* True for --numstat */
 
-  asNewFile = (diffFlags & DIFF_VERBOSE)!=0;
+  asNewFile = (diffFlags & (DIFF_VERBOSE|DIFF_NUMSTAT))!=0;
+  isNumStat = (diffFlags & DIFF_NUMSTAT)!=0;
   vid = db_lget_int("checkout", 0);
   vfile_check_signature(vid, CKSIG_ENOTFILE);
   blob_zero(&sql);
@@ -414,6 +448,7 @@ static void diff_against_disk(
     );
   }
   db_prepare(&q, "%s", blob_sql_text(&sql));
+  blob_reset(&sql);
   while( db_step(&q)==SQLITE_ROW ){
     const char *zPathname = db_column_text(&q,0);
     int isDeleted = db_column_int(&q, 1);
@@ -435,21 +470,21 @@ static void diff_against_disk(
     }
     zFullName = blob_str(&fname);
     if( isDeleted ){
-      fossil_print("DELETED  %s\n", zPathname);
+      if( !isNumStat ){ fossil_print("DELETED  %s\n", zPathname); }
       if( !asNewFile ){ showDiff = 0; zFullName = NULL_DEVICE; }
     }else if( file_access(zFullName, F_OK) ){
-      fossil_print("MISSING  %s\n", zPathname);
+      if( !isNumStat ){ fossil_print("MISSING  %s\n", zPathname); }
       if( !asNewFile ){ showDiff = 0; }
     }else if( isNew ){
-      fossil_print("ADDED    %s\n", zPathname);
+      if( !isNumStat ){ fossil_print("ADDED    %s\n", zPathname); }
       srcid = 0;
       if( !asNewFile ){ showDiff = 0; }
     }else if( isChnged==3 ){
-      fossil_print("ADDED_BY_MERGE %s\n", zPathname);
+      if( !isNumStat ){ fossil_print("ADDED_BY_MERGE %s\n", zPathname); }
       srcid = 0;
       if( !asNewFile ){ showDiff = 0; }
     }else if( isChnged==5 ){
-      fossil_print("ADDED_BY_INTEGRATE %s\n", zPathname);
+      if( !isNumStat ){ fossil_print("ADDED_BY_INTEGRATE %s\n", zPathname); }
       srcid = 0;
       if( !asNewFile ){ showDiff = 0; }
     }
@@ -470,7 +505,7 @@ static void diff_against_disk(
       isBin = fIncludeBinary ? 0 : looks_like_binary(&content);
       diff_print_index(zPathname, diffFlags);
       diff_file(&content, isBin, zFullName, zPathname, zDiffCmd,
-                zBinGlob, fIncludeBinary, diffFlags);
+                zBinGlob, fIncludeBinary, diffFlags, 0);
       blob_reset(&content);
     }
     blob_reset(&fname);
@@ -507,7 +542,7 @@ static void diff_against_undo(
     zFullName = mprintf("%s%s", g.zLocalRoot, zFile);
     db_column_blob(&q, 1, &content);
     diff_file(&content, 0, zFullName, zFile,
-              zDiffCmd, zBinGlob, fIncludeBinary, diffFlags);
+              zDiffCmd, zBinGlob, fIncludeBinary, diffFlags, 0);
     fossil_free(zFullName);
     blob_reset(&content);
   }
@@ -587,7 +622,7 @@ static void diff_two_versions(
 ){
   Manifest *pFrom, *pTo;
   ManifestFile *pFromFile, *pToFile;
-  int asNewFlag = (diffFlags & DIFF_VERBOSE)!=0 ? 1 : 0;
+  int asNewFlag = (diffFlags & (DIFF_VERBOSE|DIFF_NUMSTAT))!=0 ? 1 : 0;
 
   pFrom = manifest_get_by_name(zFrom, 0);
   manifest_file_rewind(pFrom);
@@ -607,7 +642,9 @@ static void diff_two_versions(
     }
     if( cmp<0 ){
       if( file_dir_match(pFileDir, pFromFile->zName) ){
-        fossil_print("DELETED %s\n", pFromFile->zName);
+        if( (diffFlags & DIFF_NUMSTAT)==0 ){
+          fossil_print("DELETED %s\n", pFromFile->zName);
+        }
         if( asNewFlag ){
           diff_manifest_entry(pFromFile, 0, zDiffCmd, zBinGlob,
                               fIncludeBinary, diffFlags);
@@ -616,7 +653,9 @@ static void diff_two_versions(
       pFromFile = manifest_file_next(pFrom,0);
     }else if( cmp>0 ){
       if( file_dir_match(pFileDir, pToFile->zName) ){
-        fossil_print("ADDED   %s\n", pToFile->zName);
+        if( (diffFlags & DIFF_NUMSTAT)==0 ){
+          fossil_print("ADDED   %s\n", pToFile->zName);
+        }
         if( asNewFlag ){
           diff_manifest_entry(0, pToFile, zDiffCmd, zBinGlob,
                               fIncludeBinary, diffFlags);
@@ -737,13 +776,14 @@ void diff_tk(const char *zSubCmd, int firstArg){
 ** diff programs.
 */
 int diff_include_binary_files(void){
-  if( is_truth(find_option("diff-binary", 0, 1)) ){
-    return 1;
+  const char* zArgIncludeBinary = find_option("diff-binary", 0, 1);
+
+  /* Command line argument have priority on settings */
+  if( zArgIncludeBinary ){
+    return is_truth(zArgIncludeBinary);
+  }else{
+    return db_get_boolean("diff-binary", 1);
   }
-  if( db_get_boolean("diff-binary", 1) ){
-    return 1;
-  }
-  return 0;
 }
 
 /*
@@ -776,6 +816,9 @@ const char *diff_get_binary_glob(void){
 ** no "--to" option then the (possibly edited) files in the current check-out
 ** are used.
 **
+** The "--checkin VERSION" option shows the changes made by
+** check-in VERSION relative to its primary parent.
+**
 ** The "-i" command-line option forces the use of the internal diff logic
 ** rather than any external diff program that might be configured using
 ** the "setting" command.  If no external diff program is configured, then
@@ -795,12 +838,15 @@ const char *diff_get_binary_glob(void){
 **   --binary PATTERN           Treat files that match the glob PATTERN as binary
 **   --branch BRANCH            Show diff of all changes on BRANCH
 **   --brief                    Show filenames only
+**   --checkin VERSION          Show diff of all changes in VERSION
+**   --command PROG             External diff program - overrides "diff-command"
 **   --context|-c N             Use N lines of context
 **   --diff-binary BOOL         Include binary files when using external commands
 **   --exec-abs-paths           Force absolute path names with external commands.
 **   --exec-rel-paths           Force relative path names with external commands.
 **   --from|-r VERSION          Select VERSION as source for the diff
 **   --internal|-i              Use internal diff logic
+**   --numstat                  Show only the number of lines delete and added
 **   --side-by-side|-y          Side-by-side diff
 **   --strip-trailing-cr        Strip trailing CR
 **   --tk                       Launch a Tcl/Tk GUI for display
@@ -818,6 +864,7 @@ void diff_cmd(void){
   int verboseFlag;           /* True if -v or --verbose flag is used */
   const char *zFrom;         /* Source version number */
   const char *zTo;           /* Target version number */
+  const char *zCheckin;      /* Check-in version number */
   const char *zBranch;       /* Branch to diff */
   const char *zDiffCmd = 0;  /* External diff command. NULL for internal diff */
   const char *zBinGlob = 0;  /* Treat file names matching this as binary */
@@ -834,6 +881,7 @@ void diff_cmd(void){
   isInternDiff = find_option("internal","i",0)!=0;
   zFrom = find_option("from", "r", 1);
   zTo = find_option("to", 0, 1);
+  zCheckin = find_option("checkin", 0, 1);
   zBranch = find_option("branch", 0, 1);
   againstUndo = find_option("undo",0,0)!=0;
   diffFlags = diff_options();
@@ -842,16 +890,20 @@ void diff_cmd(void){
     verboseFlag = find_option("new-file","N",0)!=0; /* deprecated */
   }
   if( verboseFlag ) diffFlags |= DIFF_VERBOSE;
-  if( againstUndo && (zFrom!=0 || zTo!=0 || zBranch!=0) ){
-    fossil_fatal("cannot use --undo together with --from or --to or --branch");
+  if( againstUndo && ( zFrom!=0 || zTo!=0 || zCheckin!=0 || zBranch!=0) ){
+    fossil_fatal("cannot use --undo together with --from, --to, --checkin,"
+                 " or --branch");
   }
   if( zBranch ){
-    if( zTo || zFrom ){
-      fossil_fatal("cannot use --from or --to with --branch");
+    if( zTo || zFrom || zCheckin ){
+      fossil_fatal("cannot use --from, --to, or --checkin with --branch");
     }
     zTo = zBranch;
     zFrom = mprintf("root:%s", zBranch);
   }
+  if( zCheckin!=0 && ( zFrom!=0 || zTo!=0 ) ){
+    fossil_fatal("cannot use --checkin together with --from or --to");
+  }
   if( zTo==0 || againstUndo ){
     db_must_be_within_tree();
   }else if( zFrom==0 ){
@@ -860,7 +912,8 @@ void diff_cmd(void){
     db_find_and_open_repository(0, 0);
   }
   if( !isInternDiff ){
-    zDiffCmd = diff_command_external(isGDiff);
+    zDiffCmd = find_option("command", 0, 1);
+    if( zDiffCmd==0 ) zDiffCmd = diff_command_external(isGDiff);
   }
   zBinGlob = diff_get_binary_glob();
   fIncludeBinary = diff_include_binary_files();
@@ -884,6 +937,17 @@ void diff_cmd(void){
       blob_reset(&fname);
     }
   }
+  if ( zCheckin!=0 ){
+    int ridTo = name_to_typed_rid(zCheckin, "ci");
+    zTo = zCheckin;
+    zFrom = db_text(0,
+      "SELECT uuid FROM blob, plink"
+      " WHERE plink.cid=%d AND plink.isprim AND plink.pid=blob.rid",
+      ridTo);
+    if( zFrom==0 ){
+      fossil_fatal("check-in %s has no parent", zTo);
+    }
+  }
   if( againstUndo ){
     if( db_lget_int("undo_available",0)==0 ){
       fossil_print("No undo or redo is available\n");
@@ -903,7 +967,7 @@ void diff_cmd(void){
     for(i=0; pFileDir[i].zName; i++){
       if( pFileDir[i].nUsed==0
        && strcmp(pFileDir[0].zName,".")!=0
-       && !file_isdir(g.argv[i+2])
+       && !file_wd_isdir(g.argv[i+2])
       ){
         fossil_fatal("not found: '%s'", g.argv[i+2]);
       }
