@@ -87,11 +87,15 @@ static struct fossilStat fileStat;
 ** lstat() is called on Unix if isWd is TRUE and allow-symlinks setting is on.
 **
 */
-static int fossil_stat(const char *zFilename, struct fossilStat *buf, int isWd){
+static int fossil_stat(
+  const char *zFilename,  /* name of file or directory to inspect. */
+  struct fossilStat *buf, /* pointer to buffer where info should go. */
+  int isWd                /* non-zero to consider look at symlink itself. */
+){
   int rc;
   void *zMbcs = fossil_utf8_to_path(zFilename, 0);
 #if !defined(_WIN32)
-  if( isWd && g.allowSymlinks ){
+  if( isWd && db_allow_symlinks() ){
     rc = lstat(zMbcs, buf);
   }else{
     rc = stat(zMbcs, buf);
@@ -103,6 +107,14 @@ static int fossil_stat(const char *zFilename, struct fossilStat *buf, int isWd){
   return rc;
 }
 
+/*
+** Clears the fileStat variable and its associated validity flag.
+*/
+static void resetStat(){
+  fileStatValid = 0;
+  memset(&fileStat, 0, sizeof(struct fossilStat));
+}
+
 /*
 ** Fill in the fileStat variable for the file named zFilename.
 ** If zFilename==0, then use the previous value of fileStat if
@@ -160,6 +172,22 @@ i64 file_wd_mtime(const char *zFilename){
   return getStat(zFilename, 1) ? -1 : fileStat.st_mtime;
 }
 
+/*
+** Return the mode bits for a file.  Return -1 if the file does not
+** exist.  If zFilename is NULL return the size of the most recently
+** stat-ed file.
+*/
+int file_mode(const char *zFilename){
+  return getStat(zFilename, 0) ? -1 : fileStat.st_mode;
+}
+
+/*
+** Same as file_mode(), but takes into account symlinks.
+*/
+int file_wd_mode(const char *zFilename){
+  return getStat(zFilename, 1) ? -1 : fileStat.st_mode;
+}
+
 /*
 ** Return TRUE if the named file is an ordinary file or symlink
 ** and symlinks are allowed.
@@ -193,7 +221,7 @@ int file_wd_isfile(const char *zFilename){
 **/
 void symlink_create(const char *zTargetFile, const char *zLinkFile){
 #if !defined(_WIN32)
-  if( g.allowSymlinks ){
+  if( db_allow_symlinks() ){
     int i, nName;
     char *zName, zBuf[1000];
 
@@ -250,7 +278,7 @@ int file_wd_perm(const char *zFilename){
   if( !getStat(zFilename, 1) ){
      if( S_ISREG(fileStat.st_mode) && ((S_IXUSR)&fileStat.st_mode)!=0 )
       return PERM_EXE;
-    else if( g.allowSymlinks && S_ISLNK(fileStat.st_mode) )
+    else if( db_allow_symlinks() && S_ISLNK(fileStat.st_mode) )
       return PERM_LNK;
   }
 #endif
@@ -295,20 +323,32 @@ int file_isdir(const char *zFilename){
 }
 
 /*
-** Same as file_isdir(), but takes into account symlinks.
+** Same as file_isdir(), but takes into account symlinks.  Return 1 if
+** zFilename is a directory -OR- a symlink that points to a directory.
+** Return 0 if zFilename does not exist.  Return 2 if zFilename exists
+** but is something other than a directory.
 */
 int file_wd_isdir(const char *zFilename){
   int rc;
+  char *zFN;
 
-  if( zFilename ){
-    char *zFN = mprintf("%s", zFilename);
-    file_simplify_name(zFN, -1, 0);
-    rc = getStat(zFN, 1);
-    free(zFN);
+  zFN = mprintf("%s", zFilename);
+  file_simplify_name(zFN, -1, 0);
+  rc = getStat(zFN, 1);
+  if( rc ){
+    rc = 0; /* It does not exist at all. */
+  }else if( S_ISDIR(fileStat.st_mode) ){
+    rc = 1; /* It exists and is a real directory. */
+  }else if( S_ISLNK(fileStat.st_mode) ){
+    Blob content;
+    blob_read_link(&content, zFN); /* It exists and is a link. */
+    rc = file_wd_isdir(blob_str(&content)); /* Points to directory? */
+    blob_reset(&content);
   }else{
-    rc = getStat(0, 1);
+    rc = 2; /* It exists and is something else. */
   }
-  return rc ? 0 : (S_ISDIR(fileStat.st_mode) ? 1 : 2);
+  free(zFN);
+  return rc;
 }
 
 
@@ -473,12 +513,12 @@ int file_wd_setexe(const char *zFilename, int onoff){
   if( fossil_stat(zFilename, &buf, 1)!=0 || S_ISLNK(buf.st_mode) ) return 0;
   if( onoff ){
     int targetMode = (buf.st_mode & 0444)>>2;
-    if( (buf.st_mode & 0100) == 0 ){
+    if( (buf.st_mode & 0100)==0 ){
       chmod(zFilename, buf.st_mode | targetMode);
       rc = 1;
     }
   }else{
-    if( (buf.st_mode & 0100) != 0 ){
+    if( (buf.st_mode & 0100)!=0 ){
       chmod(zFilename, buf.st_mode & ~0111);
       rc = 1;
     }
@@ -523,7 +563,7 @@ void test_set_mtime(void){
   if( g.argc!=4 ){
     usage("FILENAME DATE/TIME");
   }
-  db_open_or_attach(":memory:", "mem", 0);
+  db_open_or_attach(":memory:", "mem");
   iMTime = db_int64(0, "SELECT strftime('%%s',%Q)", g.argv[3]);
   zFile = g.argv[2];
   file_set_mtime(zFile, iMTime);
@@ -585,34 +625,25 @@ int file_mkdir(const char *zName, int forceFlag){
 ** print an error message and abort.
 */
 int file_mkfolder(const char *zFilename, int forceFlag, int errorReturn){
-  int i, nName, rc = 0;
+  int nName, rc = 0;
   char *zName;
 
   nName = strlen(zFilename);
   zName = mprintf("%s", zFilename);
   nName = file_simplify_name(zName, nName, 0);
-  for(i=1; i<nName; i++){
-    if( zName[i]=='/' ){
-      zName[i] = 0;
-#if defined(_WIN32) || defined(__CYGWIN__)
-      /*
-      ** On Windows, local path looks like: C:/develop/project/file.txt
-      ** The if stops us from trying to create a directory of a drive letter
-      ** C: in this example.
-      */
-      if( !(i==2 && zName[1]==':') ){
-#endif
-        if( file_mkdir(zName, forceFlag) && file_isdir(zName)!=1 ){
-          if (errorReturn <= 0) {
+  while( nName>0 && zName[nName-1]!='/' ){ nName--; }
+  if( nName ){
+    zName[nName-1] = 0;
+    if( file_wd_isdir(zName)!=1 ){
+      rc = file_mkfolder(zName, forceFlag, errorReturn);
+      if( rc==0 ){
+        if( file_mkdir(zName, forceFlag) && file_wd_isdir(zName)!=1 ){
+          if( errorReturn <= 0 ){
             fossil_fatal_recursive("unable to create directory %s", zName);
           }
           rc = errorReturn;
-          break;
         }
-#if defined(_WIN32) || defined(__CYGWIN__)
       }
-#endif
-      zName[i] = '/';
     }
   }
   free(zName);
@@ -758,6 +789,7 @@ static int backup_dir(const char *z, int *pJ){
 */
 int file_simplify_name(char *z, int n, int slash){
   int i = 1, j;
+  assert( z!=0 );
   if( n<0 ) n = strlen(z);
 
   /* On windows and cygwin convert all \ characters to /
@@ -853,7 +885,7 @@ void file_getcwd(char *zBuf, int nBuf){
 #else
   if( getcwd(zBuf, nBuf-1)==0 ){
     if( errno==ERANGE ){
-      fossil_fatal("pwd too big: max %d\n", nBuf-1);
+      fossil_fatal("pwd too big: max %d", nBuf-1);
     }else{
       fossil_fatal("cannot find current working directory; %s",
                    strerror(errno));
@@ -923,9 +955,94 @@ void file_canonical_name(const char *zOrigName, Blob *pOut, int slash){
                                        blob_size(pOut), slash));
 }
 
+/*
+** Emits the effective or raw stat() information for the specified
+** file or directory, optionally preserving the trailing slash and
+** resetting the cached stat() information.
+*/
+static void emitFileStat(
+  const char *zPath,
+  int raw,
+  int slash,
+  int reset
+){
+  char zBuf[100];
+  Blob x;
+  memset(zBuf, 0, sizeof(zBuf));
+  blob_zero(&x);
+  file_canonical_name(zPath, &x, slash);
+  fossil_print("%s[%s] -> [%s]\n", raw ? "RAW " : "", zPath, blob_buffer(&x));
+  blob_reset(&x);
+  if( raw ){
+    int rc;
+    struct fossilStat testFileStat;
+    memset(&testFileStat, 0, sizeof(struct fossilStat));
+    rc = fossil_stat(zPath, &testFileStat, 0);
+    fossil_print("  stat_rc      = %d\n", rc);
+    sqlite3_snprintf(sizeof(zBuf), zBuf, "%lld", testFileStat.st_size);
+    fossil_print("  stat_size    = %s\n", zBuf);
+    sqlite3_snprintf(sizeof(zBuf), zBuf, "%lld", testFileStat.st_mtime);
+    fossil_print("  stat_mtime   = %s\n", zBuf);
+    fossil_print("  stat_mode    = %d\n", testFileStat.st_mode);
+    memset(&testFileStat, 0, sizeof(struct fossilStat));
+    rc = fossil_stat(zPath, &testFileStat, 1);
+    fossil_print("  l_stat_rc    = %d\n", rc);
+    sqlite3_snprintf(sizeof(zBuf), zBuf, "%lld", testFileStat.st_size);
+    fossil_print("  l_stat_size  = %s\n", zBuf);
+    sqlite3_snprintf(sizeof(zBuf), zBuf, "%lld", testFileStat.st_mtime);
+    fossil_print("  l_stat_mtime = %s\n", zBuf);
+    fossil_print("  l_stat_mode  = %d\n", testFileStat.st_mode);
+  }else{
+    if( reset ) resetStat();
+    sqlite3_snprintf(sizeof(zBuf), zBuf, "%lld", file_wd_size(zPath));
+    fossil_print("  file_size           = %s\n", zBuf);
+    sqlite3_snprintf(sizeof(zBuf), zBuf, "%lld", file_wd_mtime(zPath));
+    fossil_print("  file_mtime          = %s\n", zBuf);
+    fossil_print("  file_mode           = %d\n", file_wd_mode(zPath));
+    fossil_print("  file_isfile         = %d\n", file_wd_isfile(zPath));
+    fossil_print("  file_isfile_or_link = %d\n", file_wd_isfile_or_link(zPath));
+    fossil_print("  file_islink         = %d\n", file_wd_islink(zPath));
+    fossil_print("  file_isexe          = %d\n", file_wd_isexe(zPath));
+    fossil_print("  file_isdir          = %d\n", file_wd_isdir(zPath));
+    if( reset ) resetStat();
+  }
+}
+
+/*
+** COMMAND: test-file-environment
+**
+** Usage: %fossil test-file-environment FILENAME...
+**
+** Display the effective file handling subsystem "settings" and then
+** display file system information about the files specified, if any.
+**
+** Options:
+**
+**     --open-config        Open the configuration database first.
+**     --slash              Trailing slashes, if any, are retained.
+**     --reset              Reset cached stat() info for each file.
+*/
+void cmd_test_file_environment(void){
+  int i;
+  int slashFlag = find_option("slash",0,0)!=0;
+  int resetFlag = find_option("reset",0,0)!=0;
+  if( find_option("open-config", 0, 0)!=0 ){
+    Th_OpenConfig(1);
+  }
+  fossil_print("filenames_are_case_sensitive() = %d\n",
+               filenames_are_case_sensitive());
+  fossil_print("db_allow_symlinks_by_default() = %d\n",
+               db_allow_symlinks_by_default());
+  fossil_print("db_allow_symlinks() = %d\n", db_allow_symlinks());
+  for(i=2; i<g.argc; i++){
+    emitFileStat(g.argv[i], 1, slashFlag, resetFlag);
+    emitFileStat(g.argv[i], 0, slashFlag, resetFlag);
+  }
+}
+
 /*
 ** COMMAND: test-canonical-name
-** 
+**
 ** Usage: %fossil test-canonical-name FILENAME...
 **
 ** Test the operation of the canonical name generator.
@@ -1241,9 +1358,9 @@ void file_parse_uri(
 }
 
 /*
-** Construct a random temporary filename into zBuf[].
+** Construct a random temporary filename into pBuf starting with zPrefix.
 */
-void file_tempname(int nBuf, char *zBuf){
+void file_tempname(Blob *pBuf, const char *zPrefix){
 #if defined(_WIN32)
   const char *azDirs[] = {
      0, /* GetTempPath */
@@ -1251,8 +1368,10 @@ void file_tempname(int nBuf, char *zBuf){
      0, /* TMP */
      ".",
   };
+  char *z;
 #else
-  static const char *const azDirs[] = {
+  static const char *azDirs[] = {
+     0, /* TMPDIR */
      "/var/tmp",
      "/usr/tmp",
      "/tmp",
@@ -1264,55 +1383,77 @@ void file_tempname(int nBuf, char *zBuf){
     "abcdefghijklmnopqrstuvwxyz"
     "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
     "0123456789";
-  unsigned int i, j;
+  unsigned int i;
   const char *zDir = ".";
   int cnt = 0;
+  char zRand[16];
 
 #if defined(_WIN32)
   wchar_t zTmpPath[MAX_PATH];
 
   if( GetTempPathW(MAX_PATH, zTmpPath) ){
     azDirs[0] = fossil_path_to_utf8(zTmpPath);
+    /* Removing trailing \ from the temp path */
+    z = (char*)azDirs[0];
+    i = (int)strlen(z)-1;
+    if( i>0 && z[i]=='\\' ) z[i] = 0;
   }
 
   azDirs[1] = fossil_getenv("TEMP");
   azDirs[2] = fossil_getenv("TMP");
+#else
+  azDirs[0] = fossil_getenv("TMPDIR");
 #endif
 
-
-  for(i=0; i<sizeof(azDirs)/sizeof(azDirs[0]); i++){
+  for(i=0; i<count(azDirs); i++){
     if( azDirs[i]==0 ) continue;
     if( !file_isdir(azDirs[i]) ) continue;
     zDir = azDirs[i];
     break;
   }
 
-  /* Check that the output buffer is large enough for the temporary file
-  ** name. If it is not, return SQLITE_ERROR.
-  */
-  if( (strlen(zDir) + 17) >= (size_t)nBuf ){
-    fossil_fatal("insufficient space for temporary filename");
-  }
-
   do{
+    blob_zero(pBuf);
     if( cnt++>20 ) fossil_panic("cannot generate a temporary filename");
-    sqlite3_snprintf(nBuf-17, zBuf, "%s/", zDir);
-    j = (int)strlen(zBuf);
-    sqlite3_randomness(15, &zBuf[j]);
-    for(i=0; i<15; i++, j++){
-      zBuf[j] = (char)zChars[ ((unsigned char)zBuf[j])%(sizeof(zChars)-1) ];
+    sqlite3_randomness(15, zRand);
+    for(i=0; i<15; i++){
+      zRand[i] = (char)zChars[ ((unsigned char)zRand[i])%(sizeof(zChars)-1) ];
     }
-    zBuf[j] = 0;
-  }while( file_size(zBuf)>=0 );
+    zRand[15] = 0;
+    blob_appendf(pBuf, "%s/%s-%s.txt", zDir, zPrefix ? zPrefix : "", zRand);
+  }while( file_size(blob_str(pBuf))>=0 );
 
 #if defined(_WIN32)
   fossil_path_free((char *)azDirs[0]);
   fossil_path_free((char *)azDirs[1]);
   fossil_path_free((char *)azDirs[2]);
+  /* Change all \ characters in the windows path into / so that they can
+  ** be safely passed to a subcommand, such as by gdiff */
+  z = blob_buffer(pBuf);
+  for(i=0; z[i]; i++) if( z[i]=='\\' ) z[i] = '/';
+#else
+  fossil_path_free((char *)azDirs[0]);
 #endif
 }
 
 
+/*
+** COMMAND: test-tempname
+** Usage:  fossil test-name BASENAME ...
+**
+** Generate temporary filenames derived from BASENAME
+*/
+void file_test_tempname(void){
+  int i;
+  Blob x = BLOB_INITIALIZER;
+  for(i=2; i<g.argc; i++){
+    file_tempname(&x, g.argv[i]);
+    fossil_print("%s\n", blob_str(&x));
+    blob_reset(&x);
+  }
+}
+
+
 /*
 ** Return true if a file named zName exists and has identical content
 ** to the blob pContent.  If zName does not exist or if the content is
@@ -1386,3 +1527,57 @@ FILE *fossil_fopen(const char *zName, const char *zMode){
 #endif
   return f;
 }
+
+/*
+** Return non-NULL if zFilename contains pathname elements that
+** are reserved on Windows.  The returned string is the disallowed
+** path element.
+*/
+const char *file_is_win_reserved(const char *zPath){
+  static const char *azRes[] = { "CON", "PRN", "AUX", "NUL", "COM", "LPT" };
+  static char zReturn[5];
+  int i;
+  while( zPath[0] ){
+    for(i=0; i<count(azRes); i++){
+      if( sqlite3_strnicmp(zPath, azRes[i], 3)==0
+       && ((i>=4 && fossil_isdigit(zPath[3])
+                 && (zPath[4]=='/' || zPath[4]=='.' || zPath[4]==0))
+          || (i<4 && (zPath[3]=='/' || zPath[3]=='.' || zPath[3]==0)))
+      ){
+        sqlite3_snprintf(5,zReturn,"%.*s", i>=4 ? 4 : 3, zPath);
+        return zReturn;
+      }
+    }
+    while( zPath[0] && zPath[0]!='/' ) zPath++;
+    while( zPath[0]=='/' ) zPath++;
+  }
+  return 0;
+}
+
+/*
+** COMMAND: test-valid-for-windows
+** Usage:  fossil test-valid-for-windows FILENAME ....
+**
+** Show which filenames are not valid for Windows
+*/
+void file_test_valid_for_windows(void){
+  int i;
+  for(i=2; i<g.argc; i++){
+    fossil_print("%s %s\n", file_is_win_reserved(g.argv[i]), g.argv[i]);
+  }
+}
+
+/*
+** Remove surplus "/" characters from the beginning of a full pathname.
+** Extra leading "/" characters are benign on unix.  But on Windows
+** machines, they must be removed.  Example:  Convert "/C:/fossil/xyx.fossil"
+** into "C:/fossil/xyz.fossil". Cygwin should behave as Windows here.
+*/
+const char *file_cleanup_fullpath(const char *z){
+#if defined(_WIN32) || defined(__CYGWIN__)
+  if( z[0]=='/' && fossil_isalpha(z[1]) && z[2]==':' && z[3]=='/' ) z++;
+#else
+  while( z[0]=='/' && z[1]=='/' ) z++;
+#endif
+  return z;
+}
