@@ -126,7 +126,7 @@ void finfo_cmd(void){
 
     file_tree_name(g.argv[2], &fname, 0, 1);
     if( zRevision ){
-      historical_version_of_file(zRevision, blob_str(&fname), &record, 0,0,0,0);
+      historical_blob(zRevision, blob_str(&fname), &record, 1);
     }else{
       int rid = db_int(0, "SELECT rid FROM vfile WHERE pathname=%B %s",
                        &fname, filename_collation());
@@ -251,7 +251,6 @@ void finfo_cmd(void){
 */
 void cat_cmd(void){
   int i;
-  int rc;
   Blob content, fname;
   const char *zRev;
   db_find_and_open_repository(0, 0);
@@ -263,10 +262,7 @@ void cat_cmd(void){
   for(i=2; i<g.argc; i++){
     file_tree_name(g.argv[i], &fname, 0, 1);
     blob_zero(&content);
-    rc = historical_version_of_file(zRev, blob_str(&fname), &content, 0,0,0,2);
-    if( rc==2 ){
-      fossil_fatal("no such file: %s", g.argv[i]);
-    }
+    historical_blob(zRev, blob_str(&fname), &content, 1);
     blob_write_to_file(&content, "-");
     blob_reset(&fname);
     blob_reset(&content);
@@ -284,13 +280,20 @@ void cat_cmd(void){
 **
 ** Additional query parameters:
 **
-**    a=DATE     Only show changes after DATE
-**    b=DATE     Only show changes before DATE
+**    a=DATETIME Only show changes after DATETIME
+**    b=DATETIME Only show changes before DATETIME
 **    n=NUM      Show the first NUM changes only
 **    brbg       Background color by branch name
 **    ubg        Background color by user name
 **    ci=UUID    Ancestors of a particular check-in
+**    orig=UUID  If both ci and orig are supplied, only show those
+**                 changes on a direct path from orig to ci.
 **    showid     Show RID values for debugging
+**
+** DATETIME may be "now" or "YYYY-MM-DDTHH:MM:SS.SSS". If in
+** year-month-day form, it may be truncated, and it may also name a
+** timezone offset from UTC as "-HH:MM" (westward) or "+HH:MM"
+** (eastward). Either no timezone suffix or "Z" means UTC.
 */
 void finfo_page(void){
   Stmt q;
@@ -300,6 +303,7 @@ void finfo_page(void){
   const char *zB;
   int n;
   int baseCheckin;
+  int origCheckin = 0;
   int fnid;
   Blob title;
   Blob sql;
@@ -310,6 +314,7 @@ void finfo_page(void){
   int fDebug = atoi(PD("debug","0"));
   int fShowId = P("showid")!=0;
   Stmt qparent;
+  int iTableId = timeline_tableid();
 
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
@@ -328,10 +333,15 @@ void finfo_page(void){
     return;
   }
   if( g.perm.Admin ){
-    style_submenu_element("MLink Table", "mtab", "%R/mlink?name=%t", zFilename);
+    style_submenu_element("MLink Table", "%R/mlink?name=%t", zFilename);
   }
   if( baseCheckin ){
-    compute_direct_ancestors(baseCheckin);
+    if( P("orig")!=0 ){
+      origCheckin = name_to_typed_rid(P("orig"),"ci");
+      path_shortest_stored_in_ancestor_table(origCheckin, baseCheckin);
+    }else{
+      compute_direct_ancestors(baseCheckin);
+    }
   }
   url_add_parameter(&url, "name", zFilename);
   blob_zero(&sql);
@@ -343,16 +353,18 @@ void finfo_page(void){
     " mlink.pid,"                                    /* Parent file rid */
     " mlink.fid,"                                    /* File rid */
     " (SELECT uuid FROM blob WHERE rid=mlink.pid),"  /* Parent file uuid */
-    " (SELECT uuid FROM blob WHERE rid=mlink.fid),"  /* Current file uuid */
+    " blob.uuid,"                                    /* Current file uuid */
     " (SELECT uuid FROM blob WHERE rid=mlink.mid),"  /* Check-in uuid */
     " event.bgcolor,"                                /* Background color */
     " (SELECT value FROM tagxref WHERE tagid=%d AND tagtype>0"
                                 " AND tagxref.rid=mlink.mid)," /* Branchname */
     " mlink.mid,"                                    /* check-in ID */
-    " mlink.pfnid"                                   /* Previous filename */
-    "  FROM mlink, event"
+    " mlink.pfnid,"                                  /* Previous filename */
+    " blob.size"                                     /* File size */
+    "  FROM mlink, event, blob"
     " WHERE mlink.fnid=%d"
-    "   AND event.objid=mlink.mid",
+    "   AND event.objid=mlink.mid"
+    "   AND mlink.fid=blob.rid",
     TAG_BRANCH, fnid
   );
   if( (zA = P("a"))!=0 ){
@@ -395,8 +407,10 @@ void finfo_page(void){
   blob_zero(&title);
   if( baseCheckin ){
     char *zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", baseCheckin);
-    char *zLink = 	href("%R/info/%!S", zUuid);
-    if( n>0 ){
+    char *zLink = href("%R/info/%!S", zUuid);
+    if( origCheckin ){
+      blob_appendf(&title, "Changes to file ");
+    }else if( n>0 ){
       blob_appendf(&title, "First %d ancestors of file ", n);
     }else{
       blob_appendf(&title, "Ancestors of file ");
@@ -404,18 +418,25 @@ void finfo_page(void){
     blob_appendf(&title,"<a href='%R/finfo?name=%T'>%h</a>",
                  zFilename, zFilename);
     if( fShowId ) blob_appendf(&title, " (%d)", fnid);
-    blob_appendf(&title, " from check-in %z%S</a>", zLink, zUuid);
+    blob_append(&title, origCheckin ? " between " : " from ", -1);
+    blob_appendf(&title, "check-in %z%S</a>", zLink, zUuid);
     if( fShowId ) blob_appendf(&title, " (%d)", baseCheckin);
     fossil_free(zUuid);
+    if( origCheckin ){
+      zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", origCheckin);
+      zLink = href("%R/info/%!S", zUuid);
+      blob_appendf(&title, " and check-in %z%S</a>", zLink, zUuid);
+      fossil_free(zUuid);
+    }
   }else{
-    blob_appendf(&title, "History of files named ");
+    blob_appendf(&title, "History of ");
     hyperlinked_path(zFilename, &title, 0, "tree", "");
     if( fShowId ) blob_appendf(&title, " (%d)", fnid);
   }
   @ <h2>%b(&title)</h2>
   blob_reset(&title);
   pGraph = graph_init();
-  @ <table id="timelineTable" class="timelineTable">
+  @ <table id="timelineTable%d(iTableId)" class="timelineTable">
   if( baseCheckin ){
     db_prepare(&qparent,
       "SELECT DISTINCT pid FROM mlink"
@@ -443,6 +464,7 @@ void finfo_page(void){
     const char *zBr = db_column_text(&q, 9);
     int fmid = db_column_int(&q, 10);
     int pfnid = db_column_int(&q, 11);
+    int szFile = db_column_int(&q, 12);
     int gidx;
     char zTime[10];
     int nParent = 0;
@@ -451,7 +473,7 @@ void finfo_page(void){
     db_bind_int(&qparent, ":fid", frid);
     db_bind_int(&qparent, ":mid", fmid);
     db_bind_int(&qparent, ":fnid", fnid);
-    while( db_step(&qparent)==SQLITE_ROW && nParent<ArraySize(aParent) ){
+    while( db_step(&qparent)==SQLITE_ROW && nParent<count(aParent) ){
       aParent[nParent] = db_column_int(&qparent, 0);
       nParent++;
     }
@@ -483,17 +505,24 @@ void finfo_page(void){
       @ <td class="timelineTableCell">
     }
     if( zUuid ){
-      if( nParent==0 ){
-        @ <b>Added</b>
-      }else if( pfnid ){
-        char *zPrevName = db_text(0, "SELECT name FROM filename WHERE fnid=%d",
-                                  pfnid);
-        @ <b>Renamed</b> from
-        @ %z(href("%R/finfo?name=%t", zPrevName))%h(zPrevName)</a>
+      if( origCheckin==0 ){
+        if( nParent==0 ){
+          @ <b>Added</b>
+        }else if( pfnid ){
+          char *zPrevName = db_text(0,"SELECT name FROM filename WHERE fnid=%d",
+                                    pfnid);
+          @ <b>Renamed</b> from
+          @ %z(href("%R/finfo?name=%t", zPrevName))%h(zPrevName)</a>
+        }
       }
       @ %z(href("%R/artifact/%!S",zUuid))[%S(zUuid)]</a>
       if( fShowId ){
-        @ (%d(frid))
+        int srcId = delta_source_rid(frid);
+        if( srcId>0 ){
+          @ (%d(frid)&larr;%d(srcId))
+        }else{
+          @ (%d(frid))
+        }
       }
       @ part of check-in
     }else{
@@ -517,8 +546,9 @@ void finfo_page(void){
       @ (%d(fmid))
     }
     @ %W(zCom) (user:
-    hyperlink_to_user(zUser, zDate, "");
-    @ branch: %z(href("%R/timeline?t=%T&n=200",zBr))%h(zBr)</a>)
+    hyperlink_to_user(zUser, zDate, ",");
+    @ branch: %z(href("%R/timeline?t=%T&n=200",zBr))%h(zBr)</a>,
+    @ size: %d(szFile))
     if( g.perm.Hyperlink && zUuid ){
       const char *z = zFilename;
       @ %z(href("%R/annotate?filename=%h&checkin=%s",z,zCkin))
@@ -533,7 +563,7 @@ void finfo_page(void){
     if( fDebug & FINFO_DEBUG_MLINK ){
       int ii;
       char *zAncLink;
-      @ <br>fid=%d(frid) pid=%d(fpid) mid=%d(fmid)
+      @ <br />fid=%d(frid) pid=%d(fpid) mid=%d(fmid)
       if( nParent>0 ){
         @ parents=%d(aParent[0])
         for(ii=1; ii<nParent; ii++){
@@ -558,7 +588,7 @@ void finfo_page(void){
     }
   }
   @ </table>
-  timeline_output_graph_javascript(pGraph, 0, 1);
+  timeline_output_graph_javascript(pGraph, 0, iTableId, 1);
   style_footer();
 }
 
@@ -568,14 +598,20 @@ void finfo_page(void){
 ** URL: /mlink?ci=NAME
 **
 ** Show all MLINK table entries for a particular file, or for
-** a particular check-in.  This screen is intended for use by developers
-** in debugging Fossil.
+** a particular check-in.
+**
+** This screen is intended for use by Fossil developers to help
+** in debugging Fossil itself.  Ordinary Fossil users are not 
+** expected to know what the MLINK table is or why it is important.
+**
+** To avoid confusing ordinary users, this page is only available
+** to adminstrators.
 */
 void mlink_page(void){
   const char *zFName = P("name");
   const char *zCI = P("ci");
   Stmt q;
-  
+
   login_check_credentials();
   if( !g.perm.Admin ){ login_needed(g.anon.Admin); return; }
   style_header("MLINK Table");
@@ -610,12 +646,12 @@ void mlink_page(void){
     @ <thead><tr>
     @ <th>Date</th>
     @ <th>Check-in</th>
-    @ <th>Parent Check-in</th>
+    @ <th>Parent<br>Check-in</th>
     @ <th>Merge?</th>
     @ <th>New</th>
     @ <th>Old</th>
-    @ <th>Exe Bit?</th>
-    @ <th>Prior Name</th>
+    @ <th>Exe<br>Bit?</th>
+    @ <th>Prior<br>Name</th>
     @ </tr></thead>
     @ <tbody>
     while( db_step(&q)==SQLITE_ROW ){
@@ -631,7 +667,7 @@ void mlink_page(void){
       @ <td><a href='%R/timeline?c=%!S(zCkin)'>%s(zDate)</a></td>
       @ <td><a href='%R/info/%!S(zCkin)'>%S(zCkin)</a></td>
       if( zParent ){
-        @ <td><a href='%R/info/%!S(zPid)'>%S(zParent)</a></td>
+        @ <td><a href='%R/info/%!S(zParent)'>%S(zParent)</a></td>
       }else{
         @ <td><i>(New)</i></td>
       }
@@ -678,17 +714,17 @@ void mlink_page(void){
     );
     @ <h1>MLINK table for check-in %h(zCI)</h1>
     render_checkin_context(mid, 1);
-    @ <hr>
+    @ <hr />
     @ <div class='brlist'>
     @ <table id='mlinktable'>
     @ <thead><tr>
     @ <th>File</th>
-    @ <th>From</th>
+    @ <th>Parent<br>Check-in</th>
     @ <th>Merge?</th>
     @ <th>New</th>
     @ <th>Old</th>
-    @ <th>Exe Bit?</th>
-    @ <th>Prior Name</th>
+    @ <th>Exe<br>Bit?</th>
+    @ <th>Prior<br>Name</th>
     @ </tr></thead>
     @ <tbody>
     while( db_step(&q)==SQLITE_ROW ){
@@ -702,7 +738,7 @@ void mlink_page(void){
       @ <tr>
       @ <td><a href='%R/finfo?name=%t(zName)'>%h(zName)</a></td>
       if( zParent ){
-        @ <td><a href='%R/info/%!S(zPid)'>%S(zParent)</a></td>
+        @ <td><a href='%R/info/%!S(zParent)'>%S(zParent)</a></td>
       }else{
         @ <td><i>(New)</i></td>
       }
