@@ -26,7 +26,7 @@
 #endif
 #include <time.h>
 
-/* Two custom conversions are used to show a prefix of SHA1 hashes:
+/* Two custom conversions are used to show a prefix of artifact hashes:
 **
 **      %!S       Prefix of a length appropriate for URLs
 **      %S        Prefix of a length appropriate for human display
@@ -46,7 +46,7 @@
 #endif
 
 /*
-** Return the number of SHA1 hash digits to display.  The number is for
+** Return the number of artifact hash digits to display.  The number is for
 ** human output if the bForUrl is false and is destined for a URL if
 ** bForUrl is false.
 */
@@ -64,6 +64,13 @@ static int hashDigits(int bForUrl){
   return bForUrl ? nDigitUrl : nDigitHuman;
 }
 
+/*
+** Return the number of characters in a %S output.
+*/
+int length_of_S_display(void){
+  return hashDigits(0);
+}
+
 /*
 ** Conversion types fall into various categories as defined by the
 ** following enumeration.
@@ -160,7 +167,7 @@ static const et_info fmtinfo[] = {
   {  'p', 16, 0, etPOINTER,    0,  1 },
   {  '/',  0, 0, etPATH,       0,  0 },
 };
-#define etNINFO  (sizeof(fmtinfo)/sizeof(fmtinfo[0]))
+#define etNINFO count(fmtinfo)
 
 /*
 ** "*val" is a double such that 0.1 <= *val < 10.0
@@ -652,7 +659,7 @@ int vxprintf(
         }else if( xtype==etDYNSTRING ){
           zExtra = bufpt;
         }else if( xtype==etSTRINGID ){
-          precision = 	hashDigits(flag_altform2);
+          precision = hashDigits(flag_altform2);
         }
         length = StrNLen32(bufpt, limit);
         if( precision>=0 && precision<length ) length = precision;
@@ -877,20 +884,29 @@ static int stdoutAtBOL = 1;
 **
 ** On windows, transform the output into the current terminal encoding
 ** if the output is going to the screen.  If output is redirected into
-** a file, no translation occurs.  No translation ever occurs on unix.
+** a file, no translation occurs. Switch output mode to binary to
+** properly process line-endings, make sure to switch the mode back to
+** text when done.
+** No translation ever occurs on unix.
 */
 void fossil_puts(const char *z, int toStdErr){
+  FILE* out = (toStdErr ? stderr : stdout);
   int n = (int)strlen(z);
   if( n==0 ) return;
+  assert( toStdErr==0 || toStdErr==1 );
   if( toStdErr==0 ) stdoutAtBOL = (z[n-1]=='\n');
 #if defined(_WIN32)
   if( fossil_utf8_to_console(z, n, toStdErr) >= 0 ){
     return;
   }
+  fflush(out);
+  _setmode(_fileno(out), _O_BINARY);
+#endif
+  fwrite(z, 1, n, out);
+#if defined(_WIN32)
+  fflush(out);
+  _setmode(_fileno(out), _O_TEXT);
 #endif
-  assert( toStdErr==0 || toStdErr==1 );
-  fwrite(z, 1, n, toStdErr ? stderr : stdout);
-  fflush(toStdErr ? stderr : stdout);
 }
 
 /*
@@ -972,7 +988,7 @@ static void fossil_errorlog(const char *zFormat, ...){
   vfprintf(out, zFormat, ap);
   fprintf(out, "\n");
   va_end(ap);
-  for(i=0; i<sizeof(azEnv)/sizeof(azEnv[0]); i++){
+  for(i=0; i<count(azEnv); i++){
     char *p;
     if( (p = fossil_getenv(azEnv[i]))!=0 ){
       fprintf(out, "%s=%s\n", azEnv[i], p);
@@ -1132,7 +1148,7 @@ void fossil_warning(const char *zFormat, ...){
 }
 
 /*
-** Turn off any NL to CRNL translation on the stream given as an
+** Turn off any LF to CRLF translation on the stream given as an
 ** argument.  This is a no-op on unix but is necessary on windows.
 */
 void fossil_binary_mode(FILE *p){
