@@ -63,7 +63,6 @@ void stat_page(void){
   i64 t, fsize;
   int n, m;
   int szMax, szAvg;
-  const char *zDb;
   int brief;
   char zBuf[100];
   const char *p;
@@ -74,17 +73,18 @@ void stat_page(void){
   style_header("Repository Statistics");
   style_adunit_config(ADUNIT_RIGHT_OK);
   if( g.perm.Admin ){
-    style_submenu_element("URLs", "URLs and Checkouts", "urllist");
-    style_submenu_element("Schema", "Repository Schema", "repo_schema");
-    style_submenu_element("Web-Cache", "Web-Cache Stats", "cachestat");
+    style_submenu_element("URLs", "urllist");
+    style_submenu_element("Schema", "repo_schema");
+    style_submenu_element("Web-Cache", "cachestat");
   }
-  style_submenu_element("Activity Reports", 0, "reports");
-  style_submenu_element("SHA1 Collisions", 0, "hash-collisions");
+  style_submenu_element("Activity Reports", "reports");
+  style_submenu_element("Hash Collisions", "hash-collisions");
+  style_submenu_element("Artifacts", "bloblist");
   if( sqlite3_compileoption_used("ENABLE_DBSTAT_VTAB") ){
-    style_submenu_element("Table Sizes", 0, "repo-tabsize");
+    style_submenu_element("Table Sizes", "repo-tabsize");
   }
   if( g.perm.Admin || g.perm.Setup || db_get_boolean("test_env_enable",0) ){
-    style_submenu_element("Environment", 0, "test_env");
+    style_submenu_element("Environment", "test_env");
   }
   @ <table class="label-value">
   @ <tr><th>Repository&nbsp;Size:</th><td>
@@ -115,14 +115,34 @@ void stat_page(void){
       @ <tr><th>Compression&nbsp;Ratio:</th><td>
       if( t/fsize < 5 ){
         b = 10;
-        fsize /= 10;
+        a = t/(fsize/10);
       }else{
         b = 1;
+        a = t/fsize;
       }
-      a = t/fsize;
       @ %d(a):%d(b)
       @ </td></tr>
     }
+    if( db_table_exists("repository","unversioned") ){
+      Stmt q;
+      char zStored[100];
+      db_prepare(&q,
+        "SELECT count(*), sum(sz), sum(length(content))"
+        "  FROM unversioned"
+        " WHERE length(hash)>1"
+      );
+      if( db_step(&q)==SQLITE_ROW && (n = db_column_int(&q,0))>0 ){
+        sqlite3_int64 iStored, pct;
+        iStored = db_column_int64(&q,2);
+        pct = (iStored*100 + fsize/2)/fsize;
+        approxSizeName(sizeof(zStored), zStored, iStored);
+        @ <tr><th>Unversioned&nbsp;Files:</th><td>
+        @ %z(href("%R/uvlist"))%d(n) files</a>,
+        @ %s(zStored) compressed, %d(pct)%% of total repository space
+        @ </td></tr>
+      }
+      db_finalize(&q);
+    }
     @ <tr><th>Number&nbsp;Of&nbsp;Check-ins:</th><td>
     n = db_int(0, "SELECT count(*) FROM event WHERE type='ci' /*scan*/");
     @ %d(n)
@@ -149,26 +169,37 @@ void stat_page(void){
   @ </td></tr>
   p = db_get("project-code", 0);
   if( p ){
-    @ <tr><th>Project&nbsp;ID:</th><td>%h(p)</td></tr>
+    @ <tr><th>Project&nbsp;ID:</th>
+    @     <td>%h(p) %h(db_get("project-name",""))</td></tr>
   }
-  @ <tr><th>Server&nbsp;ID:</th><td>%h(db_get("server-code",""))</td></tr>
+  p = db_get("parent-project-code", 0);
+  if( p ){
+    @ <tr><th>Parent&nbsp;Project&nbsp;ID:</th>
+    @      <td>%h(p) %h(db_get("parent-project-name",""))</td></tr>
+  }
+  /* @ <tr><th>Server&nbsp;ID:</th><td>%h(db_get("server-code",""))</td></tr> */
   @ <tr><th>Fossil&nbsp;Version:</th><td>
   @ %h(MANIFEST_DATE) %h(MANIFEST_VERSION)
-  @ (%h(RELEASE_VERSION)) [compiled using %h(COMPILER_NAME)]
+  @ (%h(RELEASE_VERSION)) <a href='version?verbose'>(details)</a>
   @ </td></tr>
   @ <tr><th>SQLite&nbsp;Version:</th><td>%.19s(sqlite3_sourceid())
-  @ [%.10s(&sqlite3_sourceid()[20])] (%s(sqlite3_libversion()))</td></tr>
-  @ <tr><th>Schema&nbsp;Version:</th><td>%h(g.zAuxSchema)</td></tr>
+  @ [%.10s(&sqlite3_sourceid()[20])] (%s(sqlite3_libversion()))
+  @ <a href='version?verbose'>(details)</a></td></tr>
+  if( g.eHashPolicy!=HPOLICY_AUTO ){
+    @ <tr><th>Schema&nbsp;Version:</th><td>%h(g.zAuxSchema),
+    @ %s(hpolicy_name())</td></tr>
+  }else{
+    @ <tr><th>Schema&nbsp;Version:</th><td>%h(g.zAuxSchema)</td></tr>
+  }
   @ <tr><th>Repository Rebuilt:</th><td>
   @ %h(db_get_mtime("rebuilt","%Y-%m-%d %H:%M:%S","Never"))
   @ By Fossil %h(db_get("rebuilt","Unknown"))</td></tr>
   @ <tr><th>Database&nbsp;Stats:</th><td>
-  zDb = db_name("repository");
-  @ %d(db_int(0, "PRAGMA \"%w\".page_count", zDb)) pages,
-  @ %d(db_int(0, "PRAGMA \"%w\".page_size", zDb)) bytes/page,
-  @ %d(db_int(0, "PRAGMA \"%w\".freelist_count", zDb)) free pages,
-  @ %s(db_text(0, "PRAGMA \"%w\".encoding", zDb)),
-  @ %s(db_text(0, "PRAGMA \"%w\".journal_mode", zDb)) mode
+  @ %d(db_int(0, "PRAGMA repository.page_count")) pages,
+  @ %d(db_int(0, "PRAGMA repository.page_size")) bytes/page,
+  @ %d(db_int(0, "PRAGMA repository.freelist_count")) free pages,
+  @ %s(db_text(0, "PRAGMA repository.encoding")),
+  @ %s(db_text(0, "PRAGMA repository.journal_mode")) mode
   @ </td></tr>
 
   @ </table>
@@ -192,7 +223,6 @@ void dbstat_cmd(void){
   i64 t, fsize;
   int n, m;
   int szMax, szAvg;
-  const char *zDb;
   int brief;
   int omitVers;            /* Omit Fossil and SQLite version information */
   int dbCheck;             /* True for the --db-check option */
@@ -291,15 +321,14 @@ void dbstat_cmd(void){
                  sqlite3_sourceid(), &sqlite3_sourceid()[20],
                  sqlite3_libversion());
   }
-  zDb = db_name("repository");
   fossil_print("%*s%d pages, %d bytes/pg, %d free pages, "
                "%s, %s mode\n",
                colWidth, "database-stats:",
-               db_int(0, "PRAGMA \"%w\".page_count", zDb),
-               db_int(0, "PRAGMA \"%w\".page_size", zDb),
-               db_int(0, "PRAGMA \"%w\".freelist_count", zDb),
-               db_text(0, "PRAGMA \"%w\".encoding", zDb),
-               db_text(0, "PRAGMA \"%w\".journal_mode", zDb));
+               db_int(0, "PRAGMA repository.page_count"),
+               db_int(0, "PRAGMA repository.page_size"),
+               db_int(0, "PRAGMA repository.freelist_count"),
+               db_text(0, "PRAGMA repository.encoding"),
+               db_text(0, "PRAGMA repository.journal_mode"));
   if( dbCheck ){
     fossil_print("%*s%s\n", colWidth, "database-check:",
                  db_text(0, "PRAGMA quick_check(1)"));
@@ -314,26 +343,38 @@ void dbstat_cmd(void){
 void urllist_page(void){
   Stmt q;
   int cnt;
+  int showAll = P("all")!=0;
+  int nOmitted;
+  sqlite3_int64 iNow;
+  char *zRemote;
   login_check_credentials();
   if( !g.perm.Admin ){ login_needed(0); return; }
 
   style_header("URLs and Checkouts");
   style_adunit_config(ADUNIT_RIGHT_OK);
-  style_submenu_element("Stat", "Repository Stats", "stat");
-  style_submenu_element("Schema", "Repository Schema", "repo_schema");
+  style_submenu_element("Stat", "stat");
+  style_submenu_element("Schema", "repo_schema");
+  iNow = db_int64(0, "SELECT strftime('%%s','now')");
   @ <div class="section">URLs</div>
   @ <table border="0" width='100%%'>
-  db_prepare(&q, "SELECT substr(name,9), datetime(mtime,'unixepoch')"
-                 "  FROM config WHERE name GLOB 'baseurl:*' ORDER BY 2 DESC");
+  db_prepare(&q, "SELECT substr(name,9), datetime(mtime,'unixepoch'), mtime"
+                 "  FROM config WHERE name GLOB 'baseurl:*' ORDER BY 3 DESC");
   cnt = 0;
+  nOmitted = 0;
   while( db_step(&q)==SQLITE_ROW ){
-    @ <tr><td width='100%%'>%h(db_column_text(&q,0))</td>
-    @ <td><nobr>%h(db_column_text(&q,1))</nobr></td></tr>
+    if( !showAll && db_column_int64(&q,2)<(iNow - 3600*24*30) && cnt>8 ){
+      nOmitted++;
+    }else{
+      @ <tr><td width='100%%'>%h(db_column_text(&q,0))</td>
+      @ <td><nobr>%h(db_column_text(&q,1))</nobr></td></tr>
+    }
     cnt++;
   }
   db_finalize(&q);
   if( cnt==0 ){
     @ <tr><td>(none)</td>
+  }else if( nOmitted ){
+    @ <tr><td><a href="urllist?all"><i>Show %d(nOmitted) more...</i></a>
   }
   @ </table>
   @ <div class="section">Checkouts</div>
@@ -342,8 +383,11 @@ void urllist_page(void){
                  "  FROM config WHERE name GLOB 'ckout:*' ORDER BY 2 DESC");
   cnt = 0;
   while( db_step(&q)==SQLITE_ROW ){
-    @ <tr><td width='100%%'>%h(db_column_text(&q,0))</td>
-    @ <td><nobr>%h(db_column_text(&q,1))</nobr></td></tr>
+    const char *zPath = db_column_text(&q,0);
+    if( vfile_top_of_checkout(zPath) ){
+      @ <tr><td width='100%%'>%h(zPath)</td>
+      @ <td><nobr>%h(db_column_text(&q,1))</nobr></td></tr>
+    }
     cnt++;
   }
   db_finalize(&q);
@@ -351,6 +395,18 @@ void urllist_page(void){
     @ <tr><td>(none)</td>
   }
   @ </table>
+  zRemote = db_text(0, "SELECT value FROM config WHERE name='last-sync-url'");
+  if( zRemote ){
+    @ <div class="section">Last Sync URL</div>
+    if( sqlite3_strlike("http%", zRemote, 0)==0 ){
+      UrlData x;
+      url_parse_local(zRemote, URL_OMIT_USER, &x);
+      @ <p><a href='%h(x.canonical)'>%h(zRemote)</a>
+    }else{
+      @ <p>%h(zRemote)</p>
+    }
+    @ </div>
+  }
   style_footer();
 }
 
@@ -361,24 +417,86 @@ void urllist_page(void){
 */
 void repo_schema_page(void){
   Stmt q;
+  Blob sql;
+  const char *zArg = P("n");
   login_check_credentials();
   if( !g.perm.Admin ){ login_needed(0); return; }
 
   style_header("Repository Schema");
   style_adunit_config(ADUNIT_RIGHT_OK);
-  style_submenu_element("Stat", "Repository Stats", "stat");
-  style_submenu_element("URLs", "URLs and Checkouts", "urllist");
+  style_submenu_element("Stat", "stat");
+  style_submenu_element("URLs", "urllist");
   if( sqlite3_compileoption_used("ENABLE_DBSTAT_VTAB") ){
-    style_submenu_element("Table Sizes", 0, "repo-tabsize");
+    style_submenu_element("Table Sizes", "repo-tabsize");
   }
-  db_prepare(&q, "SELECT sql FROM %s.sqlite_master WHERE sql IS NOT NULL",
-             db_name("repository"));
+  blob_init(&sql,
+    "SELECT sql FROM repository.sqlite_master WHERE sql IS NOT NULL", -1);
+  if( zArg ){
+    style_submenu_element("All", "repo_schema");
+    blob_appendf(&sql, " AND (tbl_name=%Q OR name=%Q)", zArg, zArg);
+  }
+  blob_appendf(&sql, " ORDER BY tbl_name, type<>'table', name");
+  db_prepare(&q, "%s", blob_str(&sql)/*safe-for-%s*/);
+  blob_reset(&sql);
   @ <pre>
   while( db_step(&q)==SQLITE_ROW ){
     @ %h(db_column_text(&q, 0));
   }
   @ </pre>
   db_finalize(&q);
+  if( db_table_exists("repository","sqlite_stat1") ){
+    if( zArg ){
+      db_prepare(&q,
+        "SELECT tbl, idx, stat FROM repository.sqlite_stat1"
+        " WHERE tbl LIKE %Q OR idx LIKE %Q"
+        " ORDER BY tbl, idx", zArg, zArg);
+
+      @ <hr>
+      @ <pre>
+      while( db_step(&q)==SQLITE_ROW ){
+        const char *zTab = db_column_text(&q,0);
+        const char *zIdx = db_column_text(&q,1);
+        const char *zStat = db_column_text(&q,2);
+        @ INSERT INTO sqlite_stat1 VALUES('%h(zTab)','%h(zIdx)','%h(zStat)');
+      }
+      @ </pre>
+      db_finalize(&q);
+    }else{
+      style_submenu_element("Stat1","repo_stat1");
+    }
+  }
+  style_footer();
+}
+
+/*
+** WEBPAGE: repo_stat1
+**
+** Show the sqlite_stat1 table for the repository schema
+*/
+void repo_stat1_page(void){
+  login_check_credentials();
+  if( !g.perm.Admin ){ login_needed(0); return; }
+
+  style_header("Repository STAT1 Table");
+  style_adunit_config(ADUNIT_RIGHT_OK);
+  style_submenu_element("Stat", "stat");
+  style_submenu_element("Schema", "repo_schema");
+  if( db_table_exists("repository","sqlite_stat1") ){
+    Stmt q;
+    db_prepare(&q,
+      "SELECT tbl, idx, stat FROM repository.sqlite_stat1"
+      " ORDER BY tbl, idx");
+    @ <pre>
+    while( db_step(&q)==SQLITE_ROW ){
+      const char *zTab = db_column_text(&q,0);
+      const char *zIdx = db_column_text(&q,1);
+      const char *zStat = db_column_text(&q,2);
+      char *zUrl = href("%R/repo_schema?n=%t",zTab);
+      @ INSERT INTO sqlite_stat1 VALUES('%z(zUrl)%h(zTab)</a>','%h(zIdx)','%h(zStat)');
+    }
+    @ </pre>
+    db_finalize(&q);
+  }
   style_footer();
 }
 
@@ -396,24 +514,22 @@ void repo_tabsize_page(void){
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
   style_header("Repository Table Sizes");
   style_adunit_config(ADUNIT_RIGHT_OK);
-  style_submenu_element("Stat", "Repository Stats", "stat");
+  style_submenu_element("Stat", "stat");
   if( g.perm.Admin ){
-    style_submenu_element("Schema", "Repository Schema", "repo_schema");
+    style_submenu_element("Schema", "repo_schema");
   }
   db_multi_exec(
-    "CREATE VIRTUAL TABLE temp.dbx USING dbstat(%s);"
     "CREATE TEMP TABLE trans(name TEXT PRIMARY KEY,tabname TEXT)WITHOUT ROWID;"
     "INSERT INTO trans(name,tabname)"
-    "   SELECT name, tbl_name FROM %s.sqlite_master;"
+    "   SELECT name, tbl_name FROM repository.sqlite_master;"
     "CREATE TEMP TABLE piechart(amt REAL, label TEXT);"
     "INSERT INTO piechart(amt,label)"
     "  SELECT count(*), "
-    "    coalesce((SELECT tabname FROM trans WHERE trans.name=dbx.name),name)"
-    "    FROM dbx"
-    "   GROUP BY 2 ORDER BY 2;",
-    db_name("repository"), db_name("repository")
+    "  coalesce((SELECT tabname FROM trans WHERE trans.name=dbstat.name),name)"
+    "    FROM dbstat('repository')"
+    "   GROUP BY 2 ORDER BY 2;"
   );
-  nPageFree = db_int(0, "PRAGMA \"%w\".freelist_count", db_name("repository"));
+  nPageFree = db_int(0, "PRAGMA repository.freelist_count");
   if( nPageFree>0 ){
     db_multi_exec(
       "INSERT INTO piechart(amt,label) VALUES(%d,'freelist')",
@@ -429,20 +545,17 @@ void repo_tabsize_page(void){
 
   if( g.localOpen ){
     db_multi_exec(
-      "DROP TABLE temp.dbx;"
-      "CREATE VIRTUAL TABLE temp.dbx USING dbstat(%s);"
       "DELETE FROM trans;"
       "INSERT INTO trans(name,tabname)"
-      "   SELECT name, tbl_name FROM %s.sqlite_master;"
+      "   SELECT name, tbl_name FROM localdb.sqlite_master;"
       "DELETE FROM piechart;"
       "INSERT INTO piechart(amt,label)"
       "  SELECT count(*), "
-      "    coalesce((SELECT tabname FROM trans WHERE trans.name=dbx.name),name)"
-      "    FROM dbx"
-      "   GROUP BY 2 ORDER BY 2;",
-      db_name("localdb"), db_name("localdb")
+      " coalesce((SELECT tabname FROM trans WHERE trans.name=dbstat.name),name)"
+      "    FROM dbstat('localdb')"
+      "   GROUP BY 2 ORDER BY 2;"
     );
-    nPageFree = db_int(0, "PRAGMA \"%s\".freelist_count", db_name("localdb"));
+    nPageFree = db_int(0, "PRAGMA localdb.freelist_count");
     if( nPageFree>0 ){
       db_multi_exec(
         "INSERT INTO piechart(amt,label) VALUES(%d,'freelist')",
