@@ -32,7 +32,9 @@
 **      style_submenu_element()
 **      style_submenu_entry()
 **      style_submenu_checkbox()
+**      style_submenu_binary()
 **      style_submenu_multichoice()
+**      style_submenu_sql()
 **
 ** prior to calling style_footer().  The style_footer() routine
 ** will generate the appropriate HTML text just below the main
@@ -40,23 +42,24 @@
 */
 static struct Submenu {
   const char *zLabel;        /* Button label */
-  const char *zTitle;
   const char *zLink;         /* Jump to this link when button is pressed */
 } aSubmenu[30];
 static int nSubmenu = 0;     /* Number of buttons */
 static struct SubmenuCtrl {
-  const char *zName;         /* Form query parameter */
-  const char *zLabel;        /* Label.  Might be NULL for FF_MULTI */
-  unsigned char eType;       /* FF_ENTRY, FF_MULTI, FF_BINARY */
-  unsigned char isDisabled;  /* True if this control is grayed out */
-  short int iSize;           /* Width for FF_ENTRY.  Count for FF_MULTI */
-  const char *const *azChoice;/* value/display pairs for FF_MULTI */
-  const char *zFalse;        /* FF_BINARY label when false */
+  const char *zName;           /* Form query parameter */
+  const char *zLabel;          /* Label.  Might be NULL for FF_MULTI */
+  unsigned char eType;         /* FF_ENTRY, FF_MULTI, FF_BINARY */
+  unsigned char isDisabled;    /* True if this control is grayed out */
+  short int iSize;             /* Width for FF_ENTRY.  Count for FF_MULTI */
+  const char *const *azChoice; /* value/display pairs for FF_MULTI */
+  const char *zFalse;          /* FF_BINARY label when false */
+  const char *zJS;             /* Javascript to run on toggle */
 } aSubmenuCtrl[20];
 static int nSubmenuCtrl = 0;
-#define FF_ENTRY  1
-#define FF_MULTI  2
-#define FF_BINARY 3
+#define FF_ENTRY    1
+#define FF_MULTI    2
+#define FF_BINARY   3
+#define FF_CHECKBOX 4
 
 /*
 ** Remember that the header has been generated.  The footer is omitted
@@ -232,14 +235,12 @@ void style_resolve_href(void){
 */
 void style_submenu_element(
   const char *zLabel,
-  const char *zTitle,
   const char *zLink,
   ...
 ){
   va_list ap;
-  assert( nSubmenu < sizeof(aSubmenu)/sizeof(aSubmenu[0]) );
+  assert( nSubmenu < count(aSubmenu) );
   aSubmenu[nSubmenu].zLabel = zLabel;
-  aSubmenu[nSubmenu].zTitle = zTitle ? zTitle : zLabel;
   va_start(ap, zLink);
   aSubmenu[nSubmenu].zLink = vmprintf(zLink, ap);
   va_end(ap);
@@ -251,7 +252,7 @@ void style_submenu_entry(
   int iSize,               /* Size of the entry box */
   int isDisabled           /* True if disabled */
 ){
-  assert( nSubmenuCtrl < ArraySize(aSubmenuCtrl) );
+  assert( nSubmenuCtrl < count(aSubmenuCtrl) );
   aSubmenuCtrl[nSubmenuCtrl].zName = zName;
   aSubmenuCtrl[nSubmenuCtrl].zLabel = zLabel;
   aSubmenuCtrl[nSubmenuCtrl].iSize = iSize;
@@ -259,13 +260,27 @@ void style_submenu_entry(
   aSubmenuCtrl[nSubmenuCtrl].eType = FF_ENTRY;
   nSubmenuCtrl++;
 }
+void style_submenu_checkbox(
+  const char *zName,       /* Query parameter name */
+  const char *zLabel,      /* Label to display after the checkbox */
+  int isDisabled,          /* True if disabled */
+  const char *zJS          /* Optional javascript to run on toggle */
+){
+  assert( nSubmenuCtrl < count(aSubmenuCtrl) );
+  aSubmenuCtrl[nSubmenuCtrl].zName = zName;
+  aSubmenuCtrl[nSubmenuCtrl].zLabel = zLabel;
+  aSubmenuCtrl[nSubmenuCtrl].isDisabled = isDisabled;
+  aSubmenuCtrl[nSubmenuCtrl].zJS = zJS;
+  aSubmenuCtrl[nSubmenuCtrl].eType = FF_CHECKBOX;
+  nSubmenuCtrl++;
+}
 void style_submenu_binary(
   const char *zName,       /* Query parameter name */
   const char *zTrue,       /* Label to show when parameter is true */
   const char *zFalse,      /* Label to show when the parameter is false */
   int isDisabled           /* True if this control is disabled */
 ){
-  assert( nSubmenuCtrl < ArraySize(aSubmenuCtrl) );
+  assert( nSubmenuCtrl < count(aSubmenuCtrl) );
   aSubmenuCtrl[nSubmenuCtrl].zName = zName;
   aSubmenuCtrl[nSubmenuCtrl].zLabel = zTrue;
   aSubmenuCtrl[nSubmenuCtrl].zFalse = zFalse;
@@ -279,7 +294,7 @@ void style_submenu_multichoice(
   const char *const *azChoice,/* value/display pairs.  2*nChoice entries */
   int isDisabled           /* True if this control is disabled */
 ){
-  assert( nSubmenuCtrl < ArraySize(aSubmenuCtrl) );
+  assert( nSubmenuCtrl < count(aSubmenuCtrl) );
   aSubmenuCtrl[nSubmenuCtrl].zName = zName;
   aSubmenuCtrl[nSubmenuCtrl].iSize = nChoice;
   aSubmenuCtrl[nSubmenuCtrl].azChoice = azChoice;
@@ -402,6 +417,7 @@ void style_header(const char *zTitleFormat, ...){
 
   /* Generate the header up through the main menu */
   Th_Store("project_name", db_get("project-name","Unnamed Fossil Project"));
+  Th_Store("project_description", db_get("project-description",""));
   Th_Store("title", zTitle);
   Th_Store("baseurl", g.zBaseURL);
   Th_Store("secureurl", login_wants_https_redirect()? g.zHttpsURL: g.zBaseURL);
@@ -472,6 +488,7 @@ static const char *style_adunit_text(unsigned int *pAdFlag){
   const char *zAd = 0;
   *pAdFlag = 0;
   if( adUnitFlags & ADUNIT_OFF ) return 0;  /* Disallow ads on this page */
+  if( db_get_boolean("adunit-disable",0) ) return 0;
   if( g.perm.Admin && db_get_boolean("adunit-omit-if-admin",0) ){
     return 0;
   }
@@ -525,69 +542,75 @@ void style_footer(void){
         }
       }
     }
-    if( nSubmenuCtrl>0 ){
-      for(i=0; i<nSubmenuCtrl; i++){
-        const char *zQPN = aSubmenuCtrl[i].zName;
-        const char *zDisabled = " disabled";
-        if( !aSubmenuCtrl[i].isDisabled ){
-          zDisabled = "";
-          cgi_tag_query_parameter(zQPN);
+    for(i=0; i<nSubmenuCtrl; i++){
+      const char *zQPN = aSubmenuCtrl[i].zName;
+      const char *zDisabled = " disabled";
+      if( !aSubmenuCtrl[i].isDisabled ){
+        zDisabled = "";
+        cgi_tag_query_parameter(zQPN);
+      }
+      switch( aSubmenuCtrl[i].eType ){
+        case FF_ENTRY:
+          @ <span class='submenuctrl'>\
+          @ &nbsp;%h(aSubmenuCtrl[i].zLabel)\
+          @ <input type='text' name='%s(zQPN)' value='%h(PD(zQPN, ""))' \
+          if( aSubmenuCtrl[i].iSize<0 ){
+            @ size='%d(-aSubmenuCtrl[i].iSize)' \
+          }else if( aSubmenuCtrl[i].iSize>0 ){
+            @ size='%d(aSubmenuCtrl[i].iSize)' \
+            @ maxlength='%d(aSubmenuCtrl[i].iSize)' \
+          }
+          @ onchange='gebi("f01").submit();'%s(zDisabled)></span>
+          break;
+        case FF_MULTI: {
+          int j;
+          const char *zVal = P(zQPN);
+          if( aSubmenuCtrl[i].zLabel ){
+            @ &nbsp;%h(aSubmenuCtrl[i].zLabel)\
+          }
+          @ <select class='submenuctrl' size='1' name='%s(zQPN)' \
+          @ onchange='gebi("f01").submit();'%s(zDisabled)>
+          for(j=0; j<aSubmenuCtrl[i].iSize*2; j+=2){
+            const char *zQPV = aSubmenuCtrl[i].azChoice[j];
+            @ <option value='%h(zQPV)'\
+            if( fossil_strcmp(zVal, zQPV)==0 ){
+              @  selected\
+            }
+            @ >%h(aSubmenuCtrl[i].azChoice[j+1])</option>
+          }
+          @ </select>
+          break;
         }
-        switch( aSubmenuCtrl[i].eType ){
-          case FF_ENTRY: {
-            cgi_printf(
-               "<span class='submenuctrl'>"
-               "&nbsp;%h<input type='text' name='%s' size='%d' maxlength='%d'"
-               " value='%h'%s></span>\n",
-               aSubmenuCtrl[i].zLabel,
-               zQPN,
-               aSubmenuCtrl[i].iSize, aSubmenuCtrl[i].iSize,
-               PD(zQPN,""),
-               zDisabled
-            );
-            break;
+        case FF_BINARY: {
+          int isTrue = PB(zQPN);
+          @ <select class='submenuctrl' size='1' name='%s(zQPN)' \
+          @ onchange='gebi("f01").submit();'%s(zDisabled)>
+          @ <option value='1'\
+          if( isTrue ){
+            @  selected\
           }
-          case FF_MULTI: {
-            int j;
-            const char *zVal = P(zQPN);
-            if( aSubmenuCtrl[i].zLabel ){
-              cgi_printf("&nbsp;%h", aSubmenuCtrl[i].zLabel);
-            }
-            cgi_printf(
-               "<select class='submenuctrl' size='1' name='%s'%s "
-               "onchange='gebi(\"f01\").submit();'>\n",
-               zQPN, zDisabled
-            );
-            for(j=0; j<aSubmenuCtrl[i].iSize*2; j+=2){
-              const char *zQPV = aSubmenuCtrl[i].azChoice[j];
-              cgi_printf(
-                "<option value='%h'%s>%h</option>\n",
-                zQPV,
-                fossil_strcmp(zVal,zQPV)==0 ? " selected" : "",
-                aSubmenuCtrl[i].azChoice[j+1]
-              );
-            }
-            @ </select>
-            break;
+          @ >%h(aSubmenuCtrl[i].zLabel)</option>
+          @ <option value='0'\
+          if( !isTrue ){
+            @  selected\
           }
-          case FF_BINARY: {
-            int isTrue = PB(zQPN);
-            cgi_printf(
-               "<select class='submenuctrl' size='1' name='%s'%s "
-               "onchange='gebi(\"f01\").submit();'>\n",
-               zQPN, zDisabled
-            );
-            cgi_printf(
-              "<option value='1'%s>%h</option>\n",
-              isTrue ? " selected":"", aSubmenuCtrl[i].zLabel
-            );
-            cgi_printf(
-              "<option value='0'%s>%h</option>\n",
-              (!isTrue) ? " selected":"", aSubmenuCtrl[i].zFalse
-            );
-            @ </select>
-            break;
+          @ >%h(aSubmenuCtrl[i].zFalse)</option>
+          @ </select>
+          break;
+        }
+        case FF_CHECKBOX: {
+          @ <label class='submenuctrl submenuckbox'>\
+          @ <input type='checkbox' name='%s(zQPN)' \
+          if( PB(zQPN) ){
+            @ checked \
           }
+          if( aSubmenuCtrl[i].zJS ){
+            @ onchange='%s(aSubmenuCtrl[i].zJS)'%s(zDisabled)>\
+          }else{
+            @ onchange='gebi("f01").submit();'%s(zDisabled)>\
+          }
+          @ %h(aSubmenuCtrl[i].zLabel)</label>
+          break;
         }
       }
     }
@@ -705,17 +728,18 @@ const struct strctCssDefaults {
     @   vertical-align: top;
     @   text-align: left;
   },
-  { "tr.timelineCurrent td.timelineTableCell",
+  { "tr.timelineCurrent",
     "the format for the timeline data cell of the current checkout",
     @   padding: .1em .2em;
     @   border: 1px dashed #446979;
+    @   box-shadow: 1px 1px 4px rgba(0, 0, 0, 0.5);
   },
   { "tr.timelineSelected",
     "The row in the timeline table that contains the entry of interest",
     @   padding: .1em .2em;
     @   border: 2px solid lightgray;
     @   background-color: #ffc;
-    @   box-shadow: 4px 4px 2px #888;
+    @   box-shadow: 4px 4px 2px rgba(0, 0, 0, 0.5);
   },
   { "tr.timelineSpacer",
     "An extra row inserted to give vertical space between two rows",
@@ -1403,6 +1427,7 @@ const struct strctCssDefaults {
   { ".brlist table th",  "Branch list table headers",
     @ text-align: left;
     @ padding: 0px 1em 0.5ex 0px;
+    @ vertical-align: bottom;
   },
   { ".brlist table td",  "Branch list table headers",
     @ padding: 0px 2em 0px 0px;
@@ -1557,13 +1582,18 @@ void page_test_env(void){
   static const char *const azCgiVars[] = {
     "COMSPEC", "DOCUMENT_ROOT", "GATEWAY_INTERFACE",
     "HTTP_ACCEPT", "HTTP_ACCEPT_CHARSET", "HTTP_ACCEPT_ENCODING",
-    "HTTP_ACCEPT_LANGUAGE", "HTTP_CONNECTION", "HTTP_HOST",
+    "HTTP_ACCEPT_LANGUAGE", "HTTP_AUTHENICATION",
+    "HTTP_CONNECTION", "HTTP_HOST",
     "HTTP_USER_AGENT", "HTTP_REFERER", "PATH_INFO", "PATH_TRANSLATED",
-    "QUERY_STRING", "REMOTE_ADDR", "REMOTE_PORT", "REQUEST_METHOD",
+    "QUERY_STRING", "REMOTE_ADDR", "REMOTE_PORT",
+    "REMOTE_USER", "REQUEST_METHOD",
     "REQUEST_URI", "SCRIPT_FILENAME", "SCRIPT_NAME", "SERVER_PROTOCOL",
     "HOME", "FOSSIL_HOME", "USERNAME", "USER", "FOSSIL_USER",
     "SQLITE_TMPDIR", "TMPDIR",
-    "TEMP", "TMP", "FOSSIL_VFS"
+    "TEMP", "TMP", "FOSSIL_VFS",
+    "FOSSIL_FORCE_TICKET_MODERATION", "FOSSIL_FORCE_WIKI_MODERATION",
+    "FOSSIL_TCL_PATH", "TH1_DELETE_INTERP", "TH1_ENABLE_DOCS",
+    "TH1_ENABLE_HOOKS", "TH1_ENABLE_TCL", "REMOTE_HOST"
   };
 
   login_check_credentials();
@@ -1573,13 +1603,9 @@ void page_test_env(void){
   }
   for(i=0; i<count(azCgiVars); i++) (void)P(azCgiVars[i]);
   style_header("Environment Test");
-  showAll = atoi(PD("showall","0"));
-  if( !showAll ){
-    style_submenu_element("Show Cookies", 0, "%R/test_env?showall=1");
-  }else{
-    style_submenu_element("Hide Cookies", 0, "%R/test_env");
-  }
-  style_submenu_element("Stats", 0, "%R/stat");
+  showAll = PB("showall");
+  style_submenu_checkbox("showall", "Cookies", 0, 0);
+  style_submenu_element("Stats", "%R/stat");
 
 #if !defined(_WIN32)
   @ uid=%d(getuid()), gid=%d(getgid())<br />
@@ -1606,11 +1632,11 @@ void page_test_env(void){
   }
   @ g.zRepositoryName = %h(g.zRepositoryName)<br />
   @ load_average() = %f(load_average())<br />
-  @ <hr>
+  @ <hr />
   P("HTTP_USER_AGENT");
   cgi_print_all(showAll);
   if( showAll && blob_size(&g.httpHeader)>0 ){
-    @ <hr>
+    @ <hr />
     @ <pre>
     @ %h(blob_str(&g.httpHeader))
     @ </pre>
