@@ -28,6 +28,15 @@
 */
 #define ONE_SECOND (1.0/86400.0)
 
+/*
+** timeline mode options
+*/
+#define TIMELINE_MODE_NONE      0
+#define TIMELINE_MODE_BEFORE    1
+#define TIMELINE_MODE_AFTER     2
+#define TIMELINE_MODE_CHILDREN  3
+#define TIMELINE_MODE_PARENTS   4
+
 /*
 ** Add an appropriate tag to the output if "rid" is unpublished (private)
 */
@@ -180,7 +189,7 @@ void test_hash_color_page(void){
     }
   }
   if( cnt ){
-    @ <hr>
+    @ <hr />
   }
   @ <form method="post" action="%s(g.zTop)/hash-color-test">
   @ <p>Enter candidate branch names below and see them displayed in their
@@ -195,6 +204,14 @@ void test_hash_color_page(void){
   style_footer();
 }
 
+/*
+** Return a new timelineTable id.
+*/
+int timeline_tableid(void){
+  static int id = 0;
+  return id++;
+}
+
 /*
 ** Output a timeline in the web format given a query.  The query
 ** should return these columns:
@@ -234,6 +251,7 @@ void www_print_timeline(
   int dateFormat = 0;         /* 0: HH:MM (default) */
   int bCommentGitStyle = 0;   /* Only show comments through first blank line */
   const char *zDateFmt;
+  int iTableId = timeline_tableid();
 
   if( fossil_strcmp(g.zIpAddr, "127.0.0.1")==0 && db_open_local(0) ){
     vid = db_lget_int("checkout", 0);
@@ -252,7 +270,7 @@ void www_print_timeline(
     TAG_BRANCH
   );
 
-  @ <table id="timelineTable" class="timelineTable">
+  @ <table id="timelineTable%d(iTableId)" class="timelineTable">
   blob_zero(&comment);
   while( db_step(pQuery)==SQLITE_ROW ){
     int rid = db_column_int(pQuery, 0);
@@ -274,7 +292,7 @@ void www_print_timeline(
     if( zDate==0 ){
       zDate = "YYYY-MM-DD HH:MM:SS";  /* Something wrong with the repo */
     }
-    modPending =  moderation_pending(rid);
+    modPending = moderation_pending(rid);
     if( tagid ){
       if( modPending ) tagid = -tagid;
       if( tagid==prevTagid ){
@@ -301,7 +319,7 @@ void www_print_timeline(
     }
     if( fossil_strcmp(zType,"div")==0 ){
       if( !prevWasDivider ){
-        @ <tr><td colspan="3"><hr class="timelineMarker"/></td></tr>
+        @ <tr><td colspan="3"><hr class="timelineMarker" /></td></tr>
       }
       prevWasDivider = 1;
       continue;
@@ -383,7 +401,7 @@ void www_print_timeline(
         " ORDER BY isprim DESC /*sort*/"
       );
       db_bind_int(&qparent, ":rid", rid);
-      while( db_step(&qparent)==SQLITE_ROW && nParent<ArraySize(aParent) ){
+      while( db_step(&qparent)==SQLITE_ROW && nParent<count(aParent) ){
         aParent[nParent++] = db_column_int(&qparent, 0);
       }
       db_reset(&qparent);
@@ -431,7 +449,12 @@ void www_print_timeline(
       hyperlink_to_uuid(zUuid);
     }
     if( tmFlags & TIMELINE_SHOWRID ){
-      @ (%d(rid))
+      int srcId = delta_source_rid(rid);
+      if( srcId ){
+        @ (%d(rid)&larr;%d(srcId))
+      }else{
+        @ (%d(rid))
+      }
     }
     db_column_blob(pQuery, commentColumn, &comment);
     if( zType[0]!='c' ){
@@ -548,13 +571,18 @@ void www_print_timeline(
         const char *zNew = db_column_text(&fchngQuery, 3);
         const char *zUnpub = "";
         char *zA;
-        char zId[20];
+        char zId[40];
         if( !inUl ){
           @ <ul class="filelist">
           inUl = 1;
         }
         if( tmFlags & TIMELINE_SHOWRID ){
-          sqlite3_snprintf(sizeof(zId), zId, " (%d) ", fid);
+          int srcId = delta_source_rid(fid);
+          if( srcId ){
+            sqlite3_snprintf(sizeof(zId), zId, " (%d&larr;%d) ", fid, srcId);
+          }else{
+            sqlite3_snprintf(sizeof(zId), zId, " (%d) ", fid);
+          }
         }else{
           zId[0] = 0;
         }
@@ -566,7 +594,7 @@ void www_print_timeline(
         }
         zA = href("%R/artifact/%!S",fid?zNew:zOld);
         if( content_is_private(fid) ){
-          zUnpub =  UNPUB_TAG;
+          zUnpub = UNPUB_TAG;
         }
         if( isNew ){
           @ <li> %s(zA)%h(zFilename)</a>%s(zId) %s(zUnpub)
@@ -616,7 +644,7 @@ void www_print_timeline(
   }
   @ </table>
   if( fchngQueryInit ) db_finalize(&fchngQuery);
-  timeline_output_graph_javascript(pGraph, (tmFlags & TIMELINE_DISJOINT)!=0, 0);
+  timeline_output_graph_javascript(pGraph, (tmFlags & TIMELINE_DISJOINT)!=0, iTableId, 0);
 }
 
 /*
@@ -657,6 +685,7 @@ static const char *bg_to_fg(const char *zIn){
 void timeline_output_graph_javascript(
   GraphContext *pGraph,     /* The graph to be displayed */
   int omitDescenders,       /* True to omit descenders */
+  int iTableId,             /* Identifier for the timelineTable */
   int fileDiff              /* True for file diff.  False for check-in diff */
 ){
   if( pGraph && pGraph->nErr==0 && pGraph->nRow>0 ){
@@ -667,6 +696,7 @@ void timeline_output_graph_javascript(
     int showArrowheads;  /* True to draw arrowheads.  False to omit. */
     int circleNodes;     /* True for circle nodes.  False for square nodes */
     int colorGraph;      /* Use colors for graph lines */
+    int iTopRow;         /* Index of the top row of the graph */
 
     iRailPitch = atoi(PD("railpitch","0"));
     showArrowheads = skin_detail_boolean("timeline-arrowheads");
@@ -692,7 +722,7 @@ void timeline_output_graph_javascript(
     **
     **   id:  The id of the <div> element for the row. This is an integer.
     **        to get an actual id, prepend "m" to the integer.  The top node
-    **        is 1 and numbers increase moving down the timeline.
+    **        is iTopRow and numbers increase moving down the timeline.
     **   bg:  The background color for this row
     **    r:  The "rail" that the node for this row sits on.  The left-most
     **        rail is 0 and the number increases to the right.
@@ -716,8 +746,9 @@ void timeline_output_graph_javascript(
     **        negative, then the rail position is the absolute value of mi[]
     **        and a thin merge-arrow descender is drawn to the bottom of
     **        the screen.
-    **    h:  The SHA1 hash of the object being graphed
+    **    h:  The artifact hash of the object being graphed
     */
+    iTopRow = pGraph->pFirst ? pGraph->pFirst->idx : 0;
     cgi_printf("var rowinfo = [\n");
     for(pRow=pGraph->pFirst; pRow; pRow=pRow->pNext){
       cgi_printf("{id:%d,bg:\"%s\",r:%d,d:%d,mo:%d,mu:%d,u:%d,f:%d,au:",
@@ -730,7 +761,7 @@ void timeline_output_graph_javascript(
         pRow->aiRiser[pRow->iRail],     /* u */
         pRow->isLeaf ? 1 : 0            /* f */
       );
-      /* u */
+      /* au */
       cSep = '[';
       for(i=0; i<GR_MAX_RAIL; i++){
         if( i==pRow->iRail ) continue;
@@ -750,7 +781,7 @@ void timeline_output_graph_javascript(
       for(i=0; i<GR_MAX_RAIL; i++){
         if( pRow->mergeIn[i] ){
           int mi = i;
-          if( pRow->mergeDown & (1<<i) ) mi = -mi;
+          if( (pRow->mergeDown >> i) & 1 ) mi = -mi;
           cgi_printf("%c%d", cSep, mi);
           cSep = ',';
         }
@@ -765,7 +796,7 @@ void timeline_output_graph_javascript(
     @ var mergeOffset;
     @ var node, arrow, arrowSmall, line, mArrow, mLine, wArrow, wLine;
     @ function initGraph(){
-    @   var parent = gebi("timelineTable").rows[0].cells[1];
+    @   var parent = gebi("timelineTable%d(iTableId)").rows[0].cells[1];
     @   parent.style.verticalAlign = "top";
     @   canvasDiv = document.createElement("div");
     @   canvasDiv.className = "tl-canvas";
@@ -894,7 +925,7 @@ void timeline_output_graph_javascript(
     @   drawBox(cls,null,x,y+(mLine.w-mArrow.h)/2);
     @ }
     @ function drawNode(p, btm){
-    @   if( p.u>0 ) drawUpArrow(p,rowinfo[p.u-1],p.fg);
+    @   if( p.u>0 ) drawUpArrow(p,rowinfo[p.u-%d(iTopRow)],p.fg);
     @   var cls = node.cls;
     @   if( p.mi.length ) cls += " merge";
     @   if( p.f&1 ) cls += " leaf";
@@ -909,7 +940,7 @@ void timeline_output_graph_javascript(
     @   if( p.mo>=0 ){
     @     var x0 = p.x + node.w/2;
     @     var x1 = p.mo*railPitch + node.w/2;
-    @     var u = rowinfo[p.mu-1];
+    @     var u = rowinfo[p.mu-%d(iTopRow)];
     @     var y1 = miLineY(u);
     @     if( p.u<0 || p.mo!=p.r ){
     @       x1 += mergeLines[p.mo] = -mLine.w/2;
@@ -933,7 +964,7 @@ void timeline_output_graph_javascript(
     @       x1 += line.w;
     @     }
     @     var y0 = p.y + (node.h-line.w)/2;
-    @     var u = rowinfo[p.au[i+1]-1];
+    @     var u = rowinfo[p.au[i+1]-%d(iTopRow)];
     @     if( u.id<p.id ){
     @       drawLine(line,u.fg,x0,y0,x1,null);
     @       drawUpArrow(p,u,u.fg);
@@ -979,7 +1010,7 @@ void timeline_output_graph_javascript(
     @ }
     @ var selRow;
     @ function clickOnNode(){
-    @   var p = rowinfo[parseInt(this.id.match(/\d+$/)[0], 10)-1];
+    @   var p = rowinfo[parseInt(this.id.match(/\d+$/)[0], 10)-%d(iTopRow)];
     @   if( !selRow ){
     @     selRow = p;
     @     this.className += " sel";
@@ -1076,7 +1107,7 @@ static void timeline_submenu(
   const char *zValue,      /* Value of the new parameter */
   const char *zRemove      /* Parameter to omit */
 ){
-  style_submenu_element(zMenuName, zMenuName, "%s",
+  style_submenu_element(zMenuName, "%s",
                         url_render(pUrl, zParam, zValue, zRemove, 0));
 }
 
@@ -1179,29 +1210,273 @@ static void timeline_y_submenu(int isDisabled){
       az[i++] = "w";
       az[i++] = "Wiki";
     }
-    assert( i<=ArraySize(az) );
+    assert( i<=count(az) );
   }
   if( i>2 ){
     style_submenu_multichoice("y", i/2, az, isDisabled);
   }
 }
 
+/*
+** If the zChng string is not NULL, then it should be a comma-separated
+** list of glob patterns for filenames.  Add an term to the WHERE clause
+** for the SQL statement under construction that excludes any check-in that
+** does not modify one or more files matching the globs.
+*/
+static void addFileGlobExclusion(
+  const char *zChng,        /* The filename GLOB list */
+  Blob *pSql                /* The SELECT statement under construction */
+){
+  if( zChng==0 || zChng[0]==0 ) return;
+  blob_append_sql(pSql," AND event.objid IN ("
+      "SELECT mlink.mid FROM mlink, filename"
+      " WHERE mlink.fnid=filename.fnid AND %s)",
+      glob_expr("filename.name", zChng));
+}
+static void addFileGlobDescription(
+  const char *zChng,        /* The filename GLOB list */
+  Blob *pDescription        /* Result description */
+){
+  if( zChng==0 || zChng[0]==0 ) return;
+  blob_appendf(pDescription, " that include changes to files matching %Q",
+               zChng);
+}
+
+/*
+** Tag match expression type code.
+*/
+typedef enum {
+  MS_EXACT,   /* Matches a single tag by exact string comparison. */
+  MS_GLOB,    /* Matches tags against a list of GLOB patterns. */
+  MS_LIKE,    /* Matches tags against a list of LIKE patterns. */
+  MS_REGEXP   /* Matches tags against a list of regular expressions. */
+} MatchStyle;
+
+/*
+** Quote a tag string by surrounding it with double quotes and preceding
+** internal double quotes and backslashes with backslashes.
+*/
+static const char *tagQuote(
+   int len,         /* Maximum length of zTag, or negative for unlimited */
+   const char *zTag /* Tag string */
+){
+  Blob blob = BLOB_INITIALIZER;
+  int i, j;
+  blob_zero(&blob);
+  blob_append(&blob, "\"", 1);
+  for( i=j=0; zTag[j] && (len<0 || j<len); ++j ){
+    if( zTag[j]=='\"' || zTag[j]=='\\' ){
+      if( j>i ){
+        blob_append(&blob, zTag+i, j-i);
+      }
+      blob_append(&blob, "\\", 1);
+      i = j;
+    }
+  }
+  if( j>i ){
+    blob_append(&blob, zTag+i, j-i);
+  }
+  blob_append(&blob, "\"", 1);
+  return blob_str(&blob);
+}
+
+/*
+** Construct the tag match SQL expression.
+**
+** This function is adapted from glob_expr() to support the MS_EXACT, MS_GLOB,
+** MS_LIKE, and MS_REGEXP match styles.  For MS_EXACT, the returned expression
+** checks for integer match against the tag ID which is looked up directly by
+** this function.  For the other modes, the returned SQL expression performs
+** string comparisons against the tag names, so it is necessary to join against
+** the tag table to access the "tagname" column.
+**
+** Each pattern is adjusted to to start with "sym-" and be anchored at end.
+**
+** In MS_REGEXP mode, backslash can be used to protect delimiter characters.
+** The backslashes are not removed from the regular expression.
+**
+** In addition to assembling and returning an SQL expression, this function
+** makes an English-language description of the patterns being matched, suitable
+** for display in the web interface.
+**
+** If any errors arise during processing, *zError is set to an error message.
+** Otherwise it is set to NULL.
+*/
+static const char *tagMatchExpression(
+  MatchStyle matchStyle,        /* Match style code */
+  const char *zTag,             /* Tag name, match pattern, or pattern list */
+  const char **zDesc,           /* Output expression description string */
+  const char **zError           /* Output error string */
+){
+  Blob expr = BLOB_INITIALIZER; /* SQL expression string assembly buffer */
+  Blob desc = BLOB_INITIALIZER; /* English description of match patterns */
+  Blob err = BLOB_INITIALIZER;  /* Error text assembly buffer */
+  const char *zStart;           /* Text at start of expression */
+  const char *zDelimiter;       /* Text between expression terms */
+  const char *zEnd;             /* Text at end of expression */
+  const char *zPrefix;          /* Text before each match pattern */
+  const char *zSuffix;          /* Text after each match pattern */
+  const char *zIntro;           /* Text introducing pattern description */
+  const char *zPattern = 0;     /* Previous quoted pattern */
+  const char *zFail = 0;        /* Current failure message or NULL if okay */
+  const char *zOr = " or ";     /* Text before final quoted pattern */
+  char cDel;                    /* Input delimiter character */
+  int i;                        /* Input match pattern length counter */
+
+  /* Optimize exact matches by looking up the ID in advance to create a simple
+   * numeric comparison.  Bypass the remainder of this function. */
+  if( matchStyle==MS_EXACT ){
+    *zDesc = tagQuote(-1, zTag);
+    return mprintf("(tagid=%d)", db_int(-1,
+        "SELECT tagid FROM tag WHERE tagname='sym-%q'", zTag));
+  }
+
+  /* Decide pattern prefix and suffix strings according to match style. */
+  if( matchStyle==MS_GLOB ){
+    zStart = "(";
+    zDelimiter = " OR ";
+    zEnd = ")";
+    zPrefix = "tagname GLOB 'sym-";
+    zSuffix = "'";
+    zIntro = "glob pattern ";
+  }else if( matchStyle==MS_LIKE ){
+    zStart = "(";
+    zDelimiter = " OR ";
+    zEnd = ")";
+    zPrefix = "tagname LIKE 'sym-";
+    zSuffix = "'";
+    zIntro = "SQL LIKE pattern ";
+  }else/* if( matchStyle==MS_REGEXP )*/{
+    zStart = "(tagname REGEXP '^sym-(";
+    zDelimiter = "|";
+    zEnd = ")$')";
+    zPrefix = "";
+    zSuffix = "";
+    zIntro = "regular expression ";
+  }
+
+  /* Convert the list of matches into an SQL expression and text description. */
+  blob_zero(&expr);
+  blob_zero(&desc);
+  blob_zero(&err);
+  while( 1 ){
+    /* Skip leading delimiters. */
+    for( ; fossil_isspace(*zTag) || *zTag==','; ++zTag );
+
+    /* Next non-delimiter character determines quoting. */
+    if( !*zTag ){
+      /* Terminate loop at end of string. */
+      break;
+    }else if( *zTag=='\'' || *zTag=='"' ){
+      /* If word is quoted, prepare to stop at end quote. */
+      cDel = *zTag;
+      ++zTag;
+    }else{
+      /* If word is not quoted, prepare to stop at delimiter. */
+      cDel = ',';
+    }
+
+    /* Find the next delimiter character or end of string. */
+    for( i=0; zTag[i] && zTag[i]!=cDel; ++i ){
+      /* If delimiter is comma, also recognize spaces as delimiters. */
+      if( cDel==',' && fossil_isspace(zTag[i]) ){
+        break;
+      }
+
+      /* In regexp mode, ignore delimiters following backslashes. */
+      if( matchStyle==MS_REGEXP && zTag[i]=='\\' && zTag[i+1] ){
+        ++i;
+      }
+    }
+
+    /* Check for regular expression syntax errors. */
+    if( matchStyle==MS_REGEXP ){
+      ReCompiled *regexp;
+      char *zTagDup = fossil_strndup(zTag, i);
+      zFail = re_compile(&regexp, zTagDup, 0);
+      re_free(regexp);
+      fossil_free(zTagDup);
+    }
+
+    /* Process success and error results. */
+    if( !zFail ){
+      /* Incorporate the match word into the output expression.  %q is used to
+       * protect against SQL injection attacks by replacing ' with ''. */
+      blob_appendf(&expr, "%s%s%#q%s", blob_size(&expr) ? zDelimiter : zStart,
+          zPrefix, i, zTag, zSuffix);
+
+      /* Build up the description string. */
+      if( !blob_size(&desc) ){
+        /* First tag: start with intro followed by first quoted tag. */
+        blob_append(&desc, zIntro, -1);
+        blob_append(&desc, tagQuote(i, zTag), -1);
+      }else{
+        if( zPattern ){
+          /* Third and subsequent tags: append comma then previous tag. */
+          blob_append(&desc, ", ", 2);
+          blob_append(&desc, zPattern, -1);
+          zOr = ", or ";
+        }
+
+        /* Second and subsequent tags: store quoted tag for next iteration. */
+        zPattern = tagQuote(i, zTag);
+      }
+    }else{
+      /* On error, skip the match word and build up the error message buffer. */
+      if( !blob_size(&err) ){
+        blob_append(&err, "Error: ", 7);
+      }else{
+        blob_append(&err, ", ", 2);
+      }
+      blob_appendf(&err, "(%s%s: %s)", zIntro, tagQuote(i, zTag), zFail);
+    }
+
+    /* Advance past all consumed input characters. */
+    zTag += i;
+    if( cDel!=',' && *zTag==cDel ){
+      ++zTag;
+    }
+  }
+
+  /* Finalize and extract the pattern description. */
+  if( zPattern ){
+    blob_append(&desc, zOr, -1);
+    blob_append(&desc, zPattern, -1);
+  }
+  *zDesc = blob_str(&desc);
+
+  /* Finalize and extract the error text. */
+  *zError = blob_size(&err) ? blob_str(&err) : 0;
+
+  /* Finalize and extract the SQL expression. */
+  if( blob_size(&expr) ){
+    blob_append(&expr, zEnd, -1);
+    return blob_str(&expr);
+  }
+
+  /* If execution reaches this point, the pattern was empty.  Return NULL. */
+  return 0;
+}
+
 /*
 ** WEBPAGE: timeline
 **
 ** Query parameters:
 **
-**    a=TIMEORTAG    after this event
-**    b=TIMEORTAG    before this event
-**    c=TIMEORTAG    "circa" this event
-**    m=TIMEORTAG    mark this event
-**    n=COUNT        suggested number of events in output
-**    p=CHECKIN      parents and ancestors of CHECKIN
-**    d=CHECKIN      descendants of CHECIN
+**    a=TIMEORTAG    After this event
+**    b=TIMEORTAG    Before this event
+**    c=TIMEORTAG    "Circa" this event
+**    m=TIMEORTAG    Mark this event
+**    n=COUNT        Suggested number of events in output
+**    p=CHECKIN      Parents and ancestors of CHECKIN
+**    d=CHECKIN      Descendants of CHECIN
 **    dp=CHECKIN     The same as d=CHECKIN&p=CHECKIN
-**    t=TAG          show only check-ins with the given TAG
-**    r=TAG          show check-ins related to TAG
-**    u=USER         only show items associated with USER
+**    t=TAG          Show only check-ins with the given TAG
+**    r=TAG          Show check-ins related to TAG, equivalent to t=TAG&rel
+**    rel            Show related check-ins as well as those matching t=TAG
+**    mionly         Limit rel to show ancestors but not descendants
+**    ms=STYLE       Set tag match style to EXACT, GLOB, LIKE, REGEXP
+**    u=USER         Only show items associated with USER
 **    y=TYPE         'ci', 'w', 't', 'e', or (default) 'all'
 **    ng             No Graph.
 **    nd             Do not highlight the focus check-in
@@ -1210,16 +1485,20 @@ static void timeline_y_submenu(int isDisabled){
 **    from=CHECKIN   Path from...
 **    to=CHECKIN       ... to this
 **    shortest         ... show only the shortest path
-**    uf=FILE_SHA1   Show only check-ins that contain the given file version
+**    uf=FILE_HASH   Show only check-ins that contain the given file version
+**    chng=GLOBLIST  Show only check-ins that involve changes to a file whose
+**                     name matches one of the comma-separate GLOBLIST
 **    brbg           Background color from branch name
 **    ubg            Background color from user
 **    namechng       Show only check-ins that have filename changes
 **    forks          Show only forks and their children
-**    ym=YYYY-MM     Show only events for the given year/month.
+**    ym=YYYY-MM     Show only events for the given year/month
 **    yw=YYYY-WW     Show only events for the given week of the given year
 **    ymd=YYYY-MM-DD Show only events on the given day
 **    datefmt=N      Override the date format
-**    bisect         Show the check-ins that are in the current bisect    
+**    bisect         Show the check-ins that are in the current bisect
+**    showid         Show RIDs
+**    showsql        Show the SQL text
 **
 ** p= and d= can appear individually or together.  If either p= or d=
 ** appear, then u=, y=, a=, and b= are ignored.
@@ -1241,17 +1520,23 @@ void page_timeline(void){
   const char *zCirca = P("c");       /* Events near this time */
   const char *zMark = P("m");        /* Mark this event or an event this time */
   const char *zTagName = P("t");     /* Show events with this tag */
-  const char *zBrName = P("r");      /* Show events related to this tag */
+  const char *zBrName = P("r");      /* Equivalent to t=TAG&rel */
+  int related = PB("rel");           /* Show events related to zTagName */
+  const char *zMatchStyle = P("ms"); /* Tag/branch match style string */
+  MatchStyle matchStyle = MS_EXACT;  /* Match style code */
+  const char *zMatchDesc = 0;        /* Tag match expression description text */
+  const char *zError = 0;            /* Tag match error string */
+  const char *zTagSql = 0;           /* Tag/branch match SQL expression */
   const char *zSearch = P("s");      /* Search string */
   const char *zUses = P("uf");       /* Only show check-ins hold this file */
   const char *zYearMonth = P("ym");  /* Show check-ins for the given YYYY-MM */
   const char *zYearWeek = P("yw");   /* Check-ins for YYYY-WW (week-of-year) */
   const char *zDay = P("ymd");       /* Check-ins for the day YYYY-MM-DD */
+  const char *zChng = P("chng");     /* List of GLOBs for files that changed */
   int useDividers = P("nd")==0;      /* Show dividers if "nd" is missing */
   int renameOnly = P("namechng")!=0; /* Show only check-ins that rename files */
   int forkOnly = PB("forks");        /* Show only forks and their children */
   int bisectOnly = PB("bisect");     /* Show the check-ins of the bisect */
-  int tagid;                         /* Tag ID */
   int tmFlags = 0;                   /* Timeline flags */
   const char *zThisTag = 0;          /* Suppress links to this tag */
   const char *zThisUser = 0;         /* Suppress links to this user */
@@ -1303,26 +1588,49 @@ void page_timeline(void){
   }
   url_initialize(&url, "timeline");
   cgi_query_parameters_to_url(&url);
-  if( zTagName && g.perm.Read ){
-    tagid = db_int(-1,"SELECT tagid FROM tag WHERE tagname='sym-%q'",zTagName);
-    zThisTag = zTagName;
-    style_submenu_element("Related", "Related", "%s",
-                          url_render(&url, "r", zTagName, "t", 0));
-  }else if( zBrName && g.perm.Read ){
-    tagid = db_int(-1,"SELECT tagid FROM tag WHERE tagname='sym-%q'",zBrName);
-    zThisTag = zBrName;
-    style_submenu_element("Branch Only", "only", "%s",
-                          url_render(&url, "t", zBrName, "r", 0));
-  }else{
-    tagid = 0;
+
+  /* Convert r=TAG to t=TAG&rel. */
+  if( zBrName && !related ){
+    cgi_delete_query_parameter("r");
+    cgi_set_query_parameter("t", zBrName);
+    cgi_set_query_parameter("rel", "1");
+    zTagName = zBrName;
+    related = 1;
   }
+
+  /* Ignore empty tag query strings. */
+  if( zTagName && !*zTagName ){
+    zTagName = 0;
+  }
+
+  /* Finish preliminary processing of tag match queries. */
+  if( zTagName ){
+    /* Interpet the tag style string. */
+    if( fossil_stricmp(zMatchStyle, "glob")==0 ){
+      matchStyle = MS_GLOB;
+    }else if( fossil_stricmp(zMatchStyle, "like")==0 ){
+      matchStyle = MS_LIKE;
+    }else if( fossil_stricmp(zMatchStyle, "regexp")==0 ){
+      matchStyle = MS_REGEXP;
+    }else{
+      /* For exact maching, inhibit links to the selected tag. */
+      zThisTag = zTagName;
+    }
+
+    /* Display a checkbox to enable/disable display of related check-ins. */
+    style_submenu_checkbox("rel", "Related", 0, 0);
+
+    /* Construct the tag match expression. */
+    zTagSql = tagMatchExpression(matchStyle, zTagName, &zMatchDesc, &zError);
+  }
+
   if( zMark && zMark[0]==0 ){
     if( zAfter ) zMark = zAfter;
     if( zBefore ) zMark = zBefore;
     if( zCirca ) zMark = zCirca;
   }
-  if( tagid
-   && db_int(0,"SELECT count(*) FROM tagxref WHERE tagid=%d",tagid)<=nEntry
+  if( (zTagSql && db_int(0,"SELECT count(*) "
+      "FROM tagxref NATURAL JOIN tag WHERE %s",zTagSql/*safe-for-%s*/)<=nEntry)
   ){
     nEntry = -1;
     zCirca = 0;
@@ -1439,13 +1747,16 @@ void page_timeline(void){
     }
     blob_append(&sql, ")", -1);
     path_reset();
+    addFileGlobExclusion(zChng, &sql);
     tmFlags |= TIMELINE_DISJOINT;
     db_multi_exec("%s", blob_sql_text(&sql));
-    blob_appendf(&desc, "%d check-ins going from ", 
+    style_submenu_checkbox("v", "Files", zType[0]!='a' && zType[0]!='c', 0);
+    blob_appendf(&desc, "%d check-ins going from ",
                  db_int(0, "SELECT count(*) FROM timeline"));
     blob_appendf(&desc, "%z[%h]</a>", href("%R/info/%h", zFrom), zFrom);
     blob_append(&desc, " to ", -1);
     blob_appendf(&desc, "%z[%h]</a>", href("%R/info/%h",zTo), zTo);
+    addFileGlobDescription(zChng, &desc);
   }else if( (p_rid || d_rid) && g.perm.Read ){
     /* If p= or d= is present, ignore all other parameters other than n= */
     char *zUuid;
@@ -1489,10 +1800,9 @@ void page_timeline(void){
         zUuid = db_text("", "SELECT uuid FROM blob WHERE rid=%d", d_rid);
       }
     }
+    style_submenu_checkbox("v", "Files", zType[0]!='a' && zType[0]!='c', 0);
     style_submenu_entry("n","Max:",4,0);
     timeline_y_submenu(1);
-    style_submenu_binary("v","With Files","Without Files",
-                         zType[0]!='a' && zType[0]!='c');
   }else if( f_rid && g.perm.Read ){
     /* If f= is present, ignore all other parameters other than n= */
     char *zUuid;
@@ -1510,11 +1820,8 @@ void page_timeline(void){
     zUuid = db_text("", "SELECT uuid FROM blob WHERE rid=%d", f_rid);
     blob_appendf(&desc, "%z[%S]</a>", href("%R/info/%!S", zUuid), zUuid);
     tmFlags |= TIMELINE_DISJOINT;
-    style_submenu_binary("v","With Files","Without Files",
-                         zType[0]!='a' && zType[0]!='c');
-    if( (tmFlags & TIMELINE_UNHIDE)==0 ){
-      timeline_submenu(&url, "Unhide", "unhide", "", 0);
-    }
+    style_submenu_checkbox("unhide", "Unhide", 0, 0);
+    style_submenu_checkbox("v", "Files", zType[0]!='a' && zType[0]!='c', 0);
   }else{
     /* Otherwise, a timeline based on a span of time */
     int n;
@@ -1522,6 +1829,10 @@ void page_timeline(void){
     char *zDate;
     Blob cond;
     blob_zero(&cond);
+    if( zChng && *zChng ){
+      addFileGlobExclusion(zChng, &cond);
+      tmFlags |= TIMELINE_DISJOINT;
+    }
     if( zUses ){
       blob_append_sql(&cond, " AND event.objid IN usesfile ");
     }
@@ -1546,12 +1857,12 @@ void page_timeline(void){
       blob_append_sql(&cond, " AND %Q=strftime('%%Y-%%m-%%d',event.mtime) ",
                    zDay);
     }
-    if( tagid ){
+    if( zTagSql ){
       blob_append_sql(&cond,
-        " AND (EXISTS(SELECT 1 FROM tagxref"
-            " WHERE tagid=%d AND tagtype>0 AND rid=blob.rid)\n", tagid);
+        " AND (EXISTS(SELECT 1 FROM tagxref NATURAL JOIN tag"
+        " WHERE %s AND tagtype>0 AND rid=blob.rid)\n", zTagSql/*safe-for-%s*/);
 
-      if( zBrName ){
+      if( related ){
         /* The next two blob_appendf() calls add SQL that causes check-ins that
         ** are not part of the branch which are parents or children of the
         ** branch to be included in the report.  This related check-ins are
@@ -1560,8 +1871,8 @@ void page_timeline(void){
         */
         blob_append_sql(&cond,
           " OR EXISTS(SELECT 1 FROM plink CROSS JOIN tagxref ON rid=cid"
-                     " WHERE tagid=%d AND tagtype>0 AND pid=blob.rid)\n",
-           tagid
+          " NATURAL JOIN tag WHERE %s AND tagtype>0 AND pid=blob.rid)\n",
+           zTagSql/*safe-for-%s*/
         );
         if( (tmFlags & TIMELINE_UNHIDE)==0 ){
           blob_append_sql(&cond,
@@ -1573,8 +1884,8 @@ void page_timeline(void){
         if( P("mionly")==0 ){
           blob_append_sql(&cond,
             " OR EXISTS(SELECT 1 FROM plink CROSS JOIN tagxref ON rid=pid"
-                       " WHERE tagid=%d AND tagtype>0 AND cid=blob.rid)\n",
-            tagid
+            " NATURAL JOIN tag WHERE %s AND tagtype>0 AND cid=blob.rid)\n",
+            zTagSql/*safe-for-%s*/
           );
           if( (tmFlags & TIMELINE_UNHIDE)==0 ){
             blob_append_sql(&cond,
@@ -1725,16 +2036,26 @@ void page_timeline(void){
       blob_appendf(&desc, " by user %h", zUser);
       tmFlags |= TIMELINE_DISJOINT;
     }
-    if( zTagName ){
-      blob_appendf(&desc, " tagged with \"%h\"", zTagName);
-      tmFlags |= TIMELINE_DISJOINT;
-    }else if( zBrName ){
-      blob_appendf(&desc, " related to \"%h\"", zBrName);
+    if( zTagSql ){
+      if( matchStyle==MS_EXACT ){
+        if( related ){
+          blob_appendf(&desc, " related to %h", zMatchDesc);
+        }else{
+          blob_appendf(&desc, " tagged with %h", zMatchDesc);
+        }
+      }else{
+        if( related ){
+          blob_appendf(&desc, " related to tags matching %h", zMatchDesc);
+        }else{
+          blob_appendf(&desc, " with tags matching %h", zMatchDesc);
+        }
+      }
       tmFlags |= TIMELINE_DISJOINT;
     }
+    addFileGlobDescription(zChng, &desc);
     if( rAfter>0.0 ){
       if( rBefore>0.0 ){
-        blob_appendf(&desc, " occurring between %h and %h.<br>",
+        blob_appendf(&desc, " occurring between %h and %h.<br />",
                      zAfter, zBefore);
       }else{
         blob_appendf(&desc, " occurring on or after %h.<br />", zAfter);
@@ -1748,6 +2069,9 @@ void page_timeline(void){
       blob_appendf(&desc, " matching \"%h\"", zSearch);
     }
     if( g.perm.Hyperlink ){
+      static const char *const azMatchStyles[] = {
+        "exact", "Exact", "glob", "Glob", "like", "Like", "regexp", "Regexp"
+      };
       double rDate;
       zDate = db_text(0, "SELECT min(timestamp) FROM timeline /*scan*/");
       if( (!zDate || !zDate[0]) && ( zAfter || zBefore ) ){
@@ -1781,14 +2105,13 @@ void page_timeline(void){
         free(zDate);
       }
       if( zType[0]=='a' || zType[0]=='c' ){
-        if( (tmFlags & TIMELINE_UNHIDE)==0 ){
-          timeline_submenu(&url, "Unhide", "unhide", "", 0);
-        }
+        style_submenu_checkbox("unhide", "Unhide", 0, 0);
       }
+      style_submenu_checkbox("v", "Files", zType[0]!='a' && zType[0]!='c', 0);
       style_submenu_entry("n","Max:",4,0);
       timeline_y_submenu(disableY);
-      style_submenu_binary("v","With Files","Without Files",
-                           zType[0]!='a' && zType[0]!='c');
+      style_submenu_entry("t", "Tag Filter:", -8, 0);
+      style_submenu_multichoice("ms", count(azMatchStyles)/2, azMatchStyles, 0);
     }
     blob_zero(&cond);
   }
@@ -1796,7 +2119,7 @@ void page_timeline(void){
     @ <pre>%h(blob_sql_text(&sql))</pre>
   }
   if( search_restrict(SRCH_CKIN)!=0 ){
-    style_submenu_element("Search", 0, "%R/search?y=c");
+    style_submenu_element("Search", "%R/search?y=c");
   }
   if( PB("showid") ) tmFlags |= TIMELINE_SHOWRID;
   if( useDividers && zMark && zMark[0] ){
@@ -1807,6 +2130,12 @@ void page_timeline(void){
   db_prepare(&q, "SELECT * FROM timeline ORDER BY sortby DESC /*scan*/");
   @ <h2>%b(&desc)</h2>
   blob_reset(&desc);
+
+  /* Report any errors. */
+  if( zError ){
+    @ <p class="generalError">%h(zError)</p>
+  }
+
   www_print_timeline(&q, tmFlags, zThisUser, zThisTag, selectedRid, 0);
   db_finalize(&q);
   if( zOlderButton ){
@@ -1997,6 +2326,13 @@ static int isIsoDate(const char *z){
       && fossil_isdigit(z[5]);
 }
 
+/*
+** Return true if the input string can be converted to a julianday.
+*/
+static int fossil_is_julianday(const char *zDate){
+  return db_int(0, "SELECT EXISTS (SELECT julianday(%Q) AS jd WHERE jd IS NOT NULL)", zDate);
+}
+
 /*
 ** COMMAND: timeline
 **
@@ -2012,14 +2348,20 @@ static int isIsoDate(const char *z){
 **     descendants | children
 **     ancestors | parents
 **
-** The CHECKIN can be any unique prefix of 4 characters or more.
-** The DATETIME should be in the ISO8601 format.  For
-** example: "2007-08-18 07:21:21".  You can also say "current"
-** for the current version or "now" for the current time.
+** The CHECKIN can be any unique prefix of 4 characters or more. You
+** can also say "current" for the current version.
+**
+** DATETIME may be "now" or "YYYY-MM-DDTHH:MM:SS.SSS". If in
+** year-month-day form, it may be truncated, the "T" may be replaced by
+** a space, and it may also name a timezone offset from UTC as "-HH:MM"
+** (westward) or "+HH:MM" (eastward). Either no timezone suffix or "Z"
+** means UTC.
+**
 **
 ** Options:
-**   -n|--limit N         Output the first N entries (default 20 lines).
-**                        N=0 means no limit.
+**   -n|--limit N         If N is positive, output the first N entries.  If
+**                        N is negative, output the first -N lines.  If N is
+**                        zero, no limit.  Default is -20 meaning 20 lines.
 **   -p|--path PATH       Output items affecting PATH only.
 **                        PATH can be a file or a sub directory.
 **   --offset P           skip P changes
@@ -2049,7 +2391,7 @@ void timeline_cmd(void){
   Blob sql;
   int objid = 0;
   Blob uuid;
-  int mode = 0 ;       /* 0:none  1: before  2:after  3:children  4:parents */
+  int mode = TIMELINE_MODE_NONE;
   int verboseFlag = 0 ;
   int iOffset;
   const char *zFilePattern = 0;
@@ -2090,17 +2432,17 @@ void timeline_cmd(void){
   if( g.argc>=4 ){
     k = strlen(g.argv[2]);
     if( strncmp(g.argv[2],"before",k)==0 ){
-      mode = 1;
+      mode = TIMELINE_MODE_BEFORE;
     }else if( strncmp(g.argv[2],"after",k)==0 && k>1 ){
-      mode = 2;
+      mode = TIMELINE_MODE_AFTER;
     }else if( strncmp(g.argv[2],"descendants",k)==0 ){
-      mode = 3;
+      mode = TIMELINE_MODE_CHILDREN;
     }else if( strncmp(g.argv[2],"children",k)==0 ){
-      mode = 3;
+      mode = TIMELINE_MODE_CHILDREN;
     }else if( strncmp(g.argv[2],"ancestors",k)==0 && k>1 ){
-      mode = 4;
+      mode = TIMELINE_MODE_PARENTS;
     }else if( strncmp(g.argv[2],"parents",k)==0 ){
-      mode = 4;
+      mode = TIMELINE_MODE_PARENTS;
     }else if(!zType && !zLimit){
       usage("?WHEN? ?CHECKIN|DATETIME? ?-n|--limit #? ?-t|--type TYPE? "
             "?-W|--width WIDTH? ?-p|--path PATH");
@@ -2119,7 +2461,7 @@ void timeline_cmd(void){
   blob_zero(&uuid);
   blob_append(&uuid, zOrigin, -1);
   if( fossil_strcmp(zOrigin, "now")==0 ){
-    if( mode==3 || mode==4 ){
+    if( mode==TIMELINE_MODE_CHILDREN || mode==TIMELINE_MODE_PARENTS ){
       fossil_fatal("cannot compute descendants or ancestors of a date");
     }
     zDate = mprintf("(SELECT datetime('now'))");
@@ -2129,18 +2471,20 @@ void timeline_cmd(void){
     }
     objid = db_lget_int("checkout",0);
     zDate = mprintf("(SELECT mtime FROM plink WHERE cid=%d)", objid);
-  }else if( name_to_uuid(&uuid, 0, "*")==0 ){
-    objid = db_int(0, "SELECT rid FROM blob WHERE uuid=%B", &uuid);
-    zDate = mprintf("(SELECT mtime FROM plink WHERE cid=%d)", objid);
-  }else{
+  }else if( fossil_is_julianday(zOrigin) ){
     const char *zShift = "";
-    if( mode==3 || mode==4 ){
+    if( mode==TIMELINE_MODE_CHILDREN || mode==TIMELINE_MODE_PARENTS ){
       fossil_fatal("cannot compute descendants or ancestors of a date");
     }
-    if( mode==0 ){
+    if( mode==TIMELINE_MODE_NONE ){
       if( isIsoDate(zOrigin) ) zShift = ",'+1 day'";
     }
     zDate = mprintf("(SELECT julianday(%Q%s, fromLocal()))", zOrigin, zShift);
+  }else if( name_to_uuid(&uuid, 0, "*")==0 ){
+    objid = db_int(0, "SELECT rid FROM blob WHERE uuid=%B", &uuid);
+    zDate = mprintf("(SELECT mtime FROM event WHERE objid=%d)", objid);
+  }else{
+    fossil_fatal("unknown check-in or invalid date: %s", zOrigin);
   }
 
   if( zFilePattern ){
@@ -2157,20 +2501,22 @@ void timeline_cmd(void){
     }
   }
 
-  if( mode==0 ) mode = 1;
+  if( mode==TIMELINE_MODE_NONE ) mode = TIMELINE_MODE_BEFORE;
   blob_zero(&sql);
   blob_append(&sql, timeline_query_for_tty(), -1);
   blob_append_sql(&sql, "\n  AND event.mtime %s %s",
-     (mode==1 || mode==4) ? "<=" : ">=",
-     zDate /*safe-for-%s*/
+     ( mode==TIMELINE_MODE_BEFORE ||
+       mode==TIMELINE_MODE_PARENTS ) ? "<=" : ">=", zDate /*safe-for-%s*/
   );
 
-  if( mode==3 || mode==4 ){
+  /* When zFilePattern is specified, compute complete ancestry;
+   * limit later at print_timeline() */
+  if( mode==TIMELINE_MODE_CHILDREN || mode==TIMELINE_MODE_PARENTS ){
     db_multi_exec("CREATE TEMP TABLE ok(rid INTEGER PRIMARY KEY)");
-    if( mode==3 ){
-      compute_descendants(objid, n);
+    if( mode==TIMELINE_MODE_CHILDREN ){
+      compute_descendants(objid, (zFilePattern ? 0 : n));
     }else{
-      compute_ancestors(objid, n, 0);
+      compute_ancestors(objid, (zFilePattern ? 0 : n), 0);
     }
     blob_append_sql(&sql, "\n  AND blob.rid IN ok");
   }
